@@ -43,12 +43,9 @@
 #include "src/core/client_channel/dynamic_filters.h"
 #include "src/core/client_channel/subchannel.h"
 #include "src/core/client_channel/subchannel_pool_interface.h"
-#include "src/core/lib/channel/call_tracer.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
 #include "src/core/lib/channel/channel_stack.h"
-#include "src/core/lib/channel/context.h"
-#include "src/core/lib/gpr/time_precise.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
@@ -60,8 +57,6 @@
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/iomgr_fwd.h"
 #include "src/core/lib/iomgr/polling_entity.h"
-#include "src/core/lib/promise/activity.h"
-#include "src/core/lib/promise/arena_promise.h"
 #include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/transport/connectivity_state.h"
@@ -71,6 +66,8 @@
 #include "src/core/load_balancing/lb_policy.h"
 #include "src/core/resolver/resolver.h"
 #include "src/core/service_config/service_config.h"
+#include "src/core/telemetry/call_tracer.h"
+#include "src/core/util/time_precise.h"
 
 //
 // Client channel filter
@@ -86,9 +83,6 @@
 // Channel arg key for server URI string.
 #define GRPC_ARG_SERVER_URI "grpc.server_uri"
 
-// Channel arg containing a pointer to the ClientChannelFilter object.
-#define GRPC_ARG_CLIENT_CHANNEL "grpc.internal.client_channel_filter"
-
 // Max number of batches that can be pending on a call at any given
 // time.  This includes one batch for each of the following ops:
 //   recv_initial_metadata
@@ -103,20 +97,16 @@ namespace grpc_core {
 
 class ClientChannelFilter final {
  public:
-  static const grpc_channel_filter kFilterVtableWithPromises;
-  static const grpc_channel_filter kFilterVtableWithoutPromises;
+  static const grpc_channel_filter kFilter;
 
   class LoadBalancedCall;
   class FilterBasedLoadBalancedCall;
-  class PromiseBasedLoadBalancedCall;
 
   // Flag that this object gets stored in channel args as a raw pointer.
   struct RawPointerChannelArgTag {};
-  static absl::string_view ChannelArgName() { return GRPC_ARG_CLIENT_CHANNEL; }
-
-  static ArenaPromise<ServerMetadataHandle> MakeCallPromise(
-      grpc_channel_element* elem, CallArgs call_args,
-      NextPromiseFactory next_promise_factory);
+  static absl::string_view ChannelArgName() {
+    return "grpc.internal.client_channel_filter";
+  }
 
   grpc_connectivity_state CheckConnectivityState(bool try_to_connect);
 
@@ -162,20 +152,9 @@ class ClientChannelFilter final {
       grpc_closure* on_call_destruction_complete,
       absl::AnyInvocable<void()> on_commit, bool is_transparent_retry);
 
-  ArenaPromise<ServerMetadataHandle> CreateLoadBalancedCallPromise(
-      CallArgs call_args, absl::AnyInvocable<void()> on_commit,
-      bool is_transparent_retry);
-
-  // Exposed for testing only.
-  static ChannelArgs MakeSubchannelArgs(
-      const ChannelArgs& channel_args, const ChannelArgs& address_args,
-      const RefCountedPtr<SubchannelPoolInterface>& subchannel_pool,
-      const std::string& channel_default_authority);
-
  private:
   class CallData;
   class FilterBasedCallData;
-  class PromiseBasedCallData;
   class ResolverResultHandler;
   class SubchannelWrapper;
   class ClientChannelControlHelper;
@@ -378,8 +357,7 @@ class ClientChannelFilter final {
 class ClientChannelFilter::LoadBalancedCall
     : public InternallyRefCounted<LoadBalancedCall, UnrefCallDtor> {
  public:
-  LoadBalancedCall(ClientChannelFilter* chand,
-                   grpc_call_context_element* call_context,
+  LoadBalancedCall(ClientChannelFilter* chand, Arena* arena,
                    absl::AnyInvocable<void()> on_commit,
                    bool is_transparent_retry);
   ~LoadBalancedCall() override;
@@ -398,8 +376,8 @@ class ClientChannelFilter::LoadBalancedCall
  protected:
   ClientChannelFilter* chand() const { return chand_; }
   ClientCallTracer::CallAttemptTracer* call_attempt_tracer() const {
-    return static_cast<ClientCallTracer::CallAttemptTracer*>(
-        call_context_[GRPC_CONTEXT_CALL_TRACER].value);
+    return DownCast<ClientCallTracer::CallAttemptTracer*>(
+        arena_->GetContext<CallTracerInterface>());
   }
   ConnectedSubchannel* connected_subchannel() const {
     return connected_subchannel_.get();
@@ -408,6 +386,7 @@ class ClientChannelFilter::LoadBalancedCall
   lb_subchannel_call_tracker() const {
     return lb_subchannel_call_tracker_.get();
   }
+  Arena* arena() const { return arena_; }
 
   void Commit() {
     auto on_commit = std::move(on_commit_);
@@ -433,14 +412,11 @@ class ClientChannelFilter::LoadBalancedCall
 
   void RecordLatency();
 
-  grpc_call_context_element* call_context() const { return call_context_; }
-
  private:
   class LbCallState;
   class Metadata;
   class BackendMetricAccessor;
 
-  virtual Arena* arena() const = 0;
   virtual grpc_polling_entity* pollent() = 0;
   virtual grpc_metadata_batch* send_initial_metadata() const = 0;
 
@@ -466,7 +442,7 @@ class ClientChannelFilter::LoadBalancedCall
   const BackendMetricData* backend_metric_data_ = nullptr;
   std::unique_ptr<LoadBalancingPolicy::SubchannelCallTrackerInterface>
       lb_subchannel_call_tracker_;
-  grpc_call_context_element* const call_context_;
+  Arena* const arena_;
 };
 
 class ClientChannelFilter::FilterBasedLoadBalancedCall final
@@ -502,7 +478,6 @@ class ClientChannelFilter::FilterBasedLoadBalancedCall final
   using LoadBalancedCall::chand;
   using LoadBalancedCall::Commit;
 
-  Arena* arena() const override { return arena_; }
   grpc_polling_entity* pollent() override { return pollent_; }
   grpc_metadata_batch* send_initial_metadata() const override {
     return pending_batches_[0]
@@ -557,7 +532,6 @@ class ClientChannelFilter::FilterBasedLoadBalancedCall final
   // TODO(roth): Instead of duplicating these fields in every filter
   // that uses any one of them, we should store them in the call
   // context.  This will save per-call memory overhead.
-  Arena* arena_;
   grpc_call_stack* owning_call_;
   CallCombiner* call_combiner_;
   grpc_polling_entity* pollent_;
@@ -594,33 +568,6 @@ class ClientChannelFilter::FilterBasedLoadBalancedCall final
   grpc_transport_stream_op_batch* pending_batches_[MAX_PENDING_BATCHES] = {};
 };
 
-class ClientChannelFilter::PromiseBasedLoadBalancedCall final
-    : public ClientChannelFilter::LoadBalancedCall {
- public:
-  PromiseBasedLoadBalancedCall(ClientChannelFilter* chand,
-                               absl::AnyInvocable<void()> on_commit,
-                               bool is_transparent_retry);
-
-  ArenaPromise<ServerMetadataHandle> MakeCallPromise(
-      CallArgs call_args, OrphanablePtr<PromiseBasedLoadBalancedCall> lb_call);
-
- private:
-  Arena* arena() const override;
-  grpc_polling_entity* pollent() override { return &pollent_; }
-  grpc_metadata_batch* send_initial_metadata() const override;
-
-  void RetryPickLocked() override;
-
-  void OnAddToQueueLocked() override
-      ABSL_EXCLUSIVE_LOCKS_REQUIRED(&ClientChannelFilter::lb_mu_);
-
-  grpc_polling_entity pollent_;
-  ClientMetadataHandle client_initial_metadata_;
-  Waker waker_;
-  bool was_queued_ = false;
-  Slice peer_string_;
-};
-
 }  // namespace grpc_core
 
 #endif  // GRPC_SRC_CORE_CLIENT_CHANNEL_CLIENT_CHANNEL_FILTER_H
