@@ -24,12 +24,11 @@
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 
 #include <grpc/status.h>
-#include <grpc/support/log.h>
 
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gpr/time_precise.h"
 #include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/gprpp/sync.h"
@@ -37,6 +36,7 @@
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
 #include "src/core/lib/transport/error_utils.h"
+#include "src/core/util/time_precise.h"
 
 #define SUBCHANNEL_STREAM_INITIAL_CONNECT_BACKOFF_SECONDS 1
 #define SUBCHANNEL_STREAM_RECONNECT_BACKOFF_MULTIPLIER 1.6
@@ -59,12 +59,13 @@ SubchannelStreamClient::SubchannelStreamClient(
       connected_subchannel_(std::move(connected_subchannel)),
       interested_parties_(interested_parties),
       tracer_(tracer),
-      call_allocator_(
+      call_allocator_(MakeRefCounted<CallArenaAllocator>(
           connected_subchannel_->args()
               .GetObject<ResourceQuota>()
               ->memory_quota()
               ->CreateMemoryAllocator(
-                  (tracer != nullptr) ? tracer : "SubchannelStreamClient")),
+                  (tracer != nullptr) ? tracer : "SubchannelStreamClient"),
+          1024)),
       event_handler_(std::move(event_handler)),
       retry_backoff_(
           BackOff::Options()
@@ -76,22 +77,22 @@ SubchannelStreamClient::SubchannelStreamClient(
                   SUBCHANNEL_STREAM_RECONNECT_MAX_BACKOFF_SECONDS))),
       event_engine_(connected_subchannel_->args().GetObject<EventEngine>()) {
   if (GPR_UNLIKELY(tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: created SubchannelStreamClient", tracer_, this);
+    LOG(INFO) << tracer_ << " " << this << ": created SubchannelStreamClient";
   }
   StartCall();
 }
 
 SubchannelStreamClient::~SubchannelStreamClient() {
   if (GPR_UNLIKELY(tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: destroying SubchannelStreamClient", tracer_,
-            this);
+    LOG(INFO) << tracer_ << " " << this
+              << ": destroying SubchannelStreamClient";
   }
 }
 
 void SubchannelStreamClient::Orphan() {
   if (GPR_UNLIKELY(tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: SubchannelStreamClient shutting down", tracer_,
-            this);
+    LOG(INFO) << tracer_ << " " << this
+              << ": SubchannelStreamClient shutting down";
   }
   {
     MutexLock lock(&mu_);
@@ -118,8 +119,9 @@ void SubchannelStreamClient::StartCallLocked() {
   }
   call_state_ = MakeOrphanable<CallState>(Ref(), interested_parties_);
   if (GPR_UNLIKELY(tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: SubchannelStreamClient created CallState %p",
-            tracer_, this, call_state_.get());
+    LOG(INFO) << tracer_ << " " << this
+              << ": SubchannelStreamClient created CallState "
+              << call_state_.get();
   }
   call_state_->StartCallLocked();
 }
@@ -130,13 +132,13 @@ void SubchannelStreamClient::StartRetryTimerLocked() {
   }
   const Duration timeout = retry_backoff_.NextAttemptTime() - Timestamp::Now();
   if (GPR_UNLIKELY(tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: SubchannelStreamClient health check call lost...",
-            tracer_, this);
+    LOG(INFO) << tracer_ << " " << this
+              << ": SubchannelStreamClient health check call lost...";
     if (timeout > Duration::Zero()) {
-      gpr_log(GPR_INFO, "%s %p: ... will retry in %" PRId64 "ms.", tracer_,
-              this, timeout.millis());
+      LOG(INFO) << tracer_ << " " << this << ": ... will retry in "
+                << timeout.millis() << "ms.";
     } else {
-      gpr_log(GPR_INFO, "%s %p: ... retrying immediately.", tracer_, this);
+      LOG(INFO) << tracer_ << " " << this << ": ... retrying immediately.";
     }
   }
   retry_timer_handle_ = event_engine_->RunAfter(
@@ -153,9 +155,8 @@ void SubchannelStreamClient::OnRetryTimer() {
   if (event_handler_ != nullptr && retry_timer_handle_.has_value() &&
       call_state_ == nullptr) {
     if (GPR_UNLIKELY(tracer_ != nullptr)) {
-      gpr_log(GPR_INFO,
-              "%s %p: SubchannelStreamClient restarting health check call",
-              tracer_, this);
+      LOG(INFO) << tracer_ << " " << this
+                << ": SubchannelStreamClient restarting health check call";
     }
     StartCallLocked();
   }
@@ -171,21 +172,13 @@ SubchannelStreamClient::CallState::CallState(
     grpc_pollset_set* interested_parties)
     : subchannel_stream_client_(std::move(health_check_client)),
       pollent_(grpc_polling_entity_create_from_pollset_set(interested_parties)),
-      arena_(Arena::Create(subchannel_stream_client_->connected_subchannel_
-                               ->GetInitialCallSizeEstimate(),
-                           &subchannel_stream_client_->call_allocator_)),
-      payload_(context_) {}
+      arena_(subchannel_stream_client_->call_allocator_->MakeArena()) {}
 
 SubchannelStreamClient::CallState::~CallState() {
   if (GPR_UNLIKELY(subchannel_stream_client_->tracer_ != nullptr)) {
-    gpr_log(GPR_INFO, "%s %p: SubchannelStreamClient destroying CallState %p",
-            subchannel_stream_client_->tracer_, subchannel_stream_client_.get(),
-            this);
-  }
-  for (size_t i = 0; i < GRPC_CONTEXT_COUNT; ++i) {
-    if (context_[i].destroy != nullptr) {
-      context_[i].destroy(context_[i].value);
-    }
+    LOG(INFO) << subchannel_stream_client_->tracer_ << " "
+              << subchannel_stream_client_.get()
+              << ": SubchannelStreamClient destroying CallState " << this;
   }
   // Unset the call combiner cancellation closure.  This has the
   // effect of scheduling the previously set cancellation closure, if
@@ -207,7 +200,6 @@ void SubchannelStreamClient::CallState::StartCallLocked() {
       gpr_get_cycle_counter(),  // start_time
       Timestamp::InfFuture(),   // deadline
       arena_.get(),
-      context_,
       &call_combiner_,
   };
   grpc_error_handle error;
@@ -218,16 +210,14 @@ void SubchannelStreamClient::CallState::StartCallLocked() {
   call_->SetAfterCallStackDestroy(&after_call_stack_destruction_);
   // Check if creation failed.
   if (!error.ok() || subchannel_stream_client_->event_handler_ == nullptr) {
-    gpr_log(GPR_ERROR,
-            "SubchannelStreamClient %p CallState %p: error creating "
-            "stream on subchannel (%s); will retry",
-            subchannel_stream_client_.get(), this,
-            StatusToString(error).c_str());
+    LOG(ERROR) << "SubchannelStreamClient " << subchannel_stream_client_.get()
+               << " CallState " << this << ": error creating "
+               << "stream on subchannel (" << StatusToString(error)
+               << "); will retry";
     CallEndedLocked(/*retry=*/true);
     return;
   }
   // Initialize payload and batch.
-  payload_.context = context_;
   batch_.payload = &payload_;
   // on_complete callback takes ref, handled manually.
   call_->Ref(DEBUG_LOCATION, "on_complete").release();
@@ -371,12 +361,10 @@ void SubchannelStreamClient::CallState::RecvMessageReady() {
               subchannel_stream_client_.get(), recv_message_->JoinIntoString());
       if (!status.ok()) {
         if (GPR_UNLIKELY(subchannel_stream_client_->tracer_ != nullptr)) {
-          gpr_log(GPR_INFO,
-                  "%s %p: SubchannelStreamClient CallState %p: failed to "
-                  "parse response message: %s",
-                  subchannel_stream_client_->tracer_,
-                  subchannel_stream_client_.get(), this,
-                  status.ToString().c_str());
+          LOG(INFO) << subchannel_stream_client_->tracer_ << " "
+                    << subchannel_stream_client_.get()
+                    << ": SubchannelStreamClient CallState " << this
+                    << ": failed to parse response message: " << status;
         }
         Cancel();
       }
@@ -419,11 +407,10 @@ void SubchannelStreamClient::CallState::RecvTrailingMetadataReady(
                           nullptr /* error_string */);
   }
   if (GPR_UNLIKELY(self->subchannel_stream_client_->tracer_ != nullptr)) {
-    gpr_log(GPR_INFO,
-            "%s %p: SubchannelStreamClient CallState %p: health watch failed "
-            "with status %d",
-            self->subchannel_stream_client_->tracer_,
-            self->subchannel_stream_client_.get(), self, status);
+    LOG(INFO) << self->subchannel_stream_client_->tracer_ << " "
+              << self->subchannel_stream_client_.get()
+              << ": SubchannelStreamClient CallState " << self
+              << ": health watch failed with status " << status;
   }
   // Clean up.
   self->recv_trailing_metadata_.Clear();
