@@ -22,6 +22,7 @@
 #include <vector>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/random/bit_gen_ref.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
@@ -29,7 +30,6 @@
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/grpc.h>
 #include <grpc/slice.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 #include "src/core/ext/transport/chaotic_good/frame.h"
@@ -97,14 +97,14 @@ ChaoticGoodServerListener::~ChaoticGoodServerListener() {
 
 absl::StatusOr<int> ChaoticGoodServerListener::Bind(
     grpc_event_engine::experimental::EventEngine::ResolvedAddress addr) {
-  if (grpc_chaotic_good_trace.enabled()) {
+  if (GRPC_TRACE_FLAG_ENABLED(chaotic_good)) {
     auto str = grpc_event_engine::experimental::ResolvedAddressToString(addr);
-    gpr_log(GPR_INFO, "CHAOTIC_GOOD: Listen on %s",
-            str.ok() ? str->c_str() : str.status().ToString().c_str());
+    LOG(INFO) << "CHAOTIC_GOOD: Listen on "
+              << (str.ok() ? str->c_str() : str.status().ToString());
   }
   EventEngine::Listener::AcceptCallback accept_cb =
-      [self = Ref()](std::unique_ptr<EventEngine::Endpoint> ep,
-                     MemoryAllocator) {
+      [self = RefAsSubclass<ChaoticGoodServerListener>()](
+          std::unique_ptr<EventEngine::Endpoint> ep, MemoryAllocator) {
         ExecCtx exec_ctx;
         MutexLock lock(&self->mu_);
         if (self->shutdown_) return;
@@ -113,8 +113,7 @@ absl::StatusOr<int> ChaoticGoodServerListener::Bind(
       };
   auto shutdown_cb = [](absl::Status status) {
     if (!status.ok()) {
-      gpr_log(GPR_ERROR, "Server accept connection failed: %s",
-              StatusToString(status).c_str());
+      LOG(ERROR) << "Server accept connection failed: " << status;
     }
   };
   CHECK_NE(event_engine_, nullptr);
@@ -123,8 +122,7 @@ absl::StatusOr<int> ChaoticGoodServerListener::Bind(
       grpc_event_engine::experimental::ChannelArgsEndpointConfig(args_),
       std::make_unique<MemoryQuota>("chaotic_good_server_listener"));
   if (!ee_listener.ok()) {
-    gpr_log(GPR_ERROR, "Bind failed: %s",
-            ee_listener.status().ToString().c_str());
+    LOG(ERROR) << "Bind failed: " << ee_listener.status().ToString();
     return ee_listener.status();
   }
   ee_listener_ = std::move(ee_listener.value());
@@ -139,9 +137,9 @@ absl::Status ChaoticGoodServerListener::StartListening() {
   CHECK(ee_listener_ != nullptr);
   auto status = ee_listener_->Start();
   if (!status.ok()) {
-    gpr_log(GPR_ERROR, "Start listening failed: %s", status.ToString().c_str());
-  } else if (grpc_chaotic_good_trace.enabled()) {
-    gpr_log(GPR_INFO, "CHAOTIC_GOOD: Started listening");
+    LOG(ERROR) << "Start listening failed: " << status;
+  } else {
+    GRPC_TRACE_LOG(chaotic_good, INFO) << "CHAOTIC_GOOD: Started listening";
   }
   return status;
 }
@@ -149,7 +147,9 @@ absl::Status ChaoticGoodServerListener::StartListening() {
 ChaoticGoodServerListener::ActiveConnection::ActiveConnection(
     RefCountedPtr<ChaoticGoodServerListener> listener,
     std::unique_ptr<EventEngine::Endpoint> endpoint)
-    : memory_allocator_(listener->memory_allocator_), listener_(listener) {
+    : listener_(std::move(listener)) {
+  arena_->SetContext<grpc_event_engine::experimental::EventEngine>(
+      listener_->event_engine_.get());
   handshaking_state_ = MakeRefCounted<HandshakingState>(Ref());
   handshaking_state_->Start(std::move(endpoint));
 }
@@ -159,9 +159,7 @@ ChaoticGoodServerListener::ActiveConnection::~ActiveConnection() {
 }
 
 void ChaoticGoodServerListener::ActiveConnection::Orphan() {
-  if (grpc_chaotic_good_trace.enabled()) {
-    gpr_log(GPR_INFO, "ActiveConnection::Orphan() %p", this);
-  }
+  GRPC_TRACE_LOG(chaotic_good, INFO) << "ActiveConnection::Orphan() " << this;
   if (handshaking_state_ != nullptr) {
     handshaking_state_->Shutdown();
     handshaking_state_.reset();
@@ -192,8 +190,7 @@ void ChaoticGoodServerListener::ActiveConnection::NewConnectionID() {
 void ChaoticGoodServerListener::ActiveConnection::Done(
     absl::optional<absl::string_view> error) {
   if (error.has_value()) {
-    gpr_log(GPR_ERROR, "ActiveConnection::Done:%p %s", this,
-            std::string(*error).c_str());
+    LOG(ERROR) << "ActiveConnection::Done:" << this << " " << *error;
   }
   // Can easily be holding various locks here: bounce through EE to ensure no
   // deadlocks.
@@ -208,16 +205,18 @@ void ChaoticGoodServerListener::ActiveConnection::Done(
 
 ChaoticGoodServerListener::ActiveConnection::HandshakingState::HandshakingState(
     RefCountedPtr<ActiveConnection> connection)
-    : memory_allocator_(connection->memory_allocator_),
-      connection_(std::move(connection)),
+    : connection_(std::move(connection)),
       handshake_mgr_(MakeRefCounted<HandshakeManager>()) {}
 
 void ChaoticGoodServerListener::ActiveConnection::HandshakingState::Start(
     std::unique_ptr<EventEngine::Endpoint> endpoint) {
   handshake_mgr_->DoHandshake(
-      grpc_event_engine_endpoint_create(std::move(endpoint)),
-      connection_->args(), GetConnectionDeadline(), nullptr, OnHandshakeDone,
-      Ref().release());
+      OrphanablePtr<grpc_endpoint>(
+          grpc_event_engine_endpoint_create(std::move(endpoint))),
+      connection_->args(), GetConnectionDeadline(), nullptr,
+      [self = Ref()](absl::StatusOr<HandshakerArgs*> result) {
+        self->OnHandshakeDone(std::move(result));
+      });
 }
 
 auto ChaoticGoodServerListener::ActiveConnection::HandshakingState::
@@ -301,11 +300,11 @@ auto ChaoticGoodServerListener::ActiveConnection::HandshakingState::
           },
           [self](PromiseEndpoint ret) -> absl::Status {
             MutexLock lock(&self->connection_->listener_->mu_);
-            if (grpc_chaotic_good_trace.enabled()) {
-              gpr_log(
-                  GPR_INFO, "%p Data endpoint setup done: shutdown=%s",
-                  self->connection_.get(),
-                  self->connection_->listener_->shutdown_ ? "true" : "false");
+            if (GRPC_TRACE_FLAG_ENABLED(chaotic_good)) {
+              LOG(INFO) << self->connection_.get()
+                        << " Data endpoint setup done: shutdown="
+                        << (self->connection_->listener_->shutdown_ ? "true"
+                                                                    : "false");
             }
             if (self->connection_->listener_->shutdown_) {
               return absl::UnavailableError("Server shutdown");
@@ -340,7 +339,10 @@ auto ChaoticGoodServerListener::ActiveConnection::HandshakingState::
       SettingsMetadata{absl::nullopt, self->connection_->connection_id_,
                        absl::nullopt}
           .ToMetadataBatch();
-  auto write_buffer = frame.Serialize(&self->connection_->hpack_compressor_);
+  bool saw_encoding_errors = false;
+  auto write_buffer = frame.Serialize(&self->connection_->hpack_compressor_,
+                                      saw_encoding_errors);
+  // ignore encoding errors: they will be logged separately already
   return TrySeq(
       self->connection_->endpoint_.Write(std::move(write_buffer.control)),
       WaitForDataEndpointSetup(self));
@@ -354,7 +356,10 @@ auto ChaoticGoodServerListener::ActiveConnection::HandshakingState::
       SettingsMetadata{absl::nullopt, self->connection_->connection_id_,
                        self->connection_->data_alignment_}
           .ToMetadataBatch();
-  auto write_buffer = frame.Serialize(&self->connection_->hpack_compressor_);
+  bool saw_encoding_errors = false;
+  auto write_buffer = frame.Serialize(&self->connection_->hpack_compressor_,
+                                      saw_encoding_errors);
+  // ignore encoding errors: they will be logged separately already
   return TrySeq(
       self->connection_->endpoint_.Write(std::move(write_buffer.control)),
       [self]() mutable {
@@ -382,33 +387,28 @@ auto ChaoticGoodServerListener::ActiveConnection::HandshakingState::
 }
 
 void ChaoticGoodServerListener::ActiveConnection::HandshakingState::
-    OnHandshakeDone(void* arg, grpc_error_handle error) {
-  auto* args = static_cast<HandshakerArgs*>(arg);
-  CHECK_NE(args, nullptr);
-  RefCountedPtr<HandshakingState> self(
-      static_cast<HandshakingState*>(args->user_data));
-  grpc_slice_buffer_destroy(args->read_buffer);
-  gpr_free(args->read_buffer);
-  if (!error.ok()) {
-    self->connection_->Done(
-        absl::StrCat("Handshake failed: ", StatusToString(error)));
+    OnHandshakeDone(absl::StatusOr<HandshakerArgs*> result) {
+  if (!result.ok()) {
+    connection_->Done(
+        absl::StrCat("Handshake failed: ", result.status().ToString()));
     return;
   }
-  if (args->endpoint == nullptr) {
-    self->connection_->Done("Server handshake done but has empty endpoint.");
+  CHECK_NE(*result, nullptr);
+  if ((*result)->endpoint == nullptr) {
+    connection_->Done("Server handshake done but has empty endpoint.");
     return;
   }
   CHECK(grpc_event_engine::experimental::grpc_is_event_engine_endpoint(
-      args->endpoint));
+      (*result)->endpoint.get()));
   auto ee_endpoint =
       grpc_event_engine::experimental::grpc_take_wrapped_event_engine_endpoint(
-          args->endpoint);
+          (*result)->endpoint.release());
   auto* chaotic_good_ext = grpc_event_engine::experimental::QueryExtension<
       grpc_event_engine::experimental::ChaoticGoodExtension>(ee_endpoint.get());
-  self->connection_->endpoint_ =
+  connection_->endpoint_ =
       PromiseEndpoint(std::move(ee_endpoint), SliceBuffer());
   auto activity = MakeActivity(
-      [self, chaotic_good_ext]() {
+      [self = Ref(), chaotic_good_ext]() {
         return TrySeq(
             Race(EndpointReadSettingsFrame(self),
                  TrySeq(Sleep(Timestamp::Now() + kConnectionDeadline),
@@ -428,8 +428,8 @@ void ChaoticGoodServerListener::ActiveConnection::HandshakingState::
               return EndpointWriteSettingsFrame(self, is_control_endpoint);
             });
       },
-      EventEngineWakeupScheduler(self->connection_->listener_->event_engine_),
-      [self](absl::Status status) {
+      EventEngineWakeupScheduler(connection_->listener_->event_engine_),
+      [self = Ref()](absl::Status status) {
         if (!status.ok()) {
           self->connection_->Done(
               absl::StrCat("Server setting frame handling failed: ",
@@ -438,28 +438,22 @@ void ChaoticGoodServerListener::ActiveConnection::HandshakingState::
           self->connection_->Done();
         }
       },
-      self->connection_->arena_.get(),
-      self->connection_->listener_->event_engine_.get());
-  MutexLock lock(&self->connection_->mu_);
-  if (self->connection_->orphaned_) return;
-  self->connection_->receive_settings_activity_ = std::move(activity);
+      connection_->arena_.get());
+  MutexLock lock(&connection_->mu_);
+  if (connection_->orphaned_) return;
+  connection_->receive_settings_activity_ = std::move(activity);
 }
 
 Timestamp ChaoticGoodServerListener::ActiveConnection::HandshakingState::
     GetConnectionDeadline() {
-  if (connection_->args().Contains(GRPC_ARG_SERVER_HANDSHAKE_TIMEOUT_MS)) {
-    return Timestamp::Now() +
-           connection_->args()
-               .GetDurationFromIntMillis(GRPC_ARG_SERVER_HANDSHAKE_TIMEOUT_MS)
-               .value();
-  }
-  return Timestamp::Now() + kConnectionDeadline;
+  return Timestamp::Now() +
+         connection_->args()
+             .GetDurationFromIntMillis(GRPC_ARG_SERVER_HANDSHAKE_TIMEOUT_MS)
+             .value_or(kConnectionDeadline);
 }
 
 void ChaoticGoodServerListener::Orphan() {
-  if (grpc_chaotic_good_trace.enabled()) {
-    gpr_log(GPR_INFO, "ChaoticGoodServerListener::Orphan()");
-  }
+  GRPC_TRACE_LOG(chaotic_good, INFO) << "ChaoticGoodServerListener::Orphan()";
   {
     absl::flat_hash_set<OrphanablePtr<ActiveConnection>> connection_list;
     MutexLock lock(&mu_);
@@ -480,25 +474,26 @@ int grpc_server_add_chaotic_good_port(grpc_server* server, const char* addr) {
   const auto resolved_or = grpc_core::GetDNSResolver()->LookupHostnameBlocking(
       parsed_addr, absl::StrCat(0xd20));
   if (!resolved_or.ok()) {
-    gpr_log(GPR_ERROR, "Failed to resolve %s: %s", addr,
-            resolved_or.status().ToString().c_str());
+    LOG(ERROR) << "Failed to resolve " << addr << ": "
+               << resolved_or.status().ToString();
     return 0;
   }
   int port_num = 0;
+  std::vector<std::pair<std::string, absl::Status>> error_list;
   for (const auto& resolved_addr : resolved_or.value()) {
     auto listener = grpc_core::MakeOrphanable<
         grpc_core::chaotic_good::ChaoticGoodServerListener>(
         core_server, core_server->channel_args());
     const auto ee_addr =
         grpc_event_engine::experimental::CreateResolvedAddress(resolved_addr);
-    gpr_log(GPR_INFO, "BIND: %s",
-            grpc_event_engine::experimental::ResolvedAddressToString(ee_addr)
-                ->c_str());
+    std::string addr_str =
+        *grpc_event_engine::experimental::ResolvedAddressToString(ee_addr);
+    GRPC_TRACE_LOG(chaotic_good, INFO) << "BIND: " << addr_str;
     auto bind_result = listener->Bind(ee_addr);
     if (!bind_result.ok()) {
-      gpr_log(GPR_ERROR, "Failed to bind to %s: %s", addr,
-              bind_result.status().ToString().c_str());
-      return 0;
+      error_list.push_back(
+          std::make_pair(std::move(addr_str), bind_result.status()));
+      continue;
     }
     if (port_num == 0) {
       port_num = bind_result.value();
@@ -507,5 +502,17 @@ int grpc_server_add_chaotic_good_port(grpc_server* server, const char* addr) {
     }
     core_server->AddListener(std::move(listener));
   }
+  if (error_list.size() == resolved_or->size()) {
+    LOG(ERROR) << "Failed to bind any address for " << addr;
+    for (const auto& error : error_list) {
+      LOG(ERROR) << "  " << error.first << ": " << error.second;
+    }
+  } else if (!error_list.empty()) {
+    LOG(INFO) << "Failed to bind some addresses for " << addr;
+    for (const auto& error : error_list) {
+      GRPC_TRACE_LOG(chaotic_good, INFO)
+          << "Binding Failed: " << error.first << ": " << error.second;
+    }
+  }
   return port_num;
 }
