@@ -27,12 +27,12 @@
 
 #include "absl/container/flat_hash_map.h"
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/types/optional.h"
 
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/slice_buffer.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
@@ -47,19 +47,14 @@
 #include "src/core/ext/transport/chttp2/transport/frame_window_update.h"
 #include "src/core/ext/transport/chttp2/transport/hpack_encoder.h"
 #include "src/core/ext/transport/chttp2/transport/http2_settings.h"
-#include "src/core/ext/transport/chttp2/transport/http_trace.h"
 #include "src/core/ext/transport/chttp2/transport/internal.h"
 #include "src/core/ext/transport/chttp2/transport/legacy_frame.h"
 #include "src/core/ext/transport/chttp2/transport/max_concurrent_streams_policy.h"
 #include "src/core/ext/transport/chttp2/transport/ping_callbacks.h"
 #include "src/core/ext/transport/chttp2/transport/ping_rate_policy.h"
 #include "src/core/ext/transport/chttp2/transport/write_size_policy.h"
-#include "src/core/lib/channel/call_tracer.h"
-#include "src/core/lib/debug/stats.h"
-#include "src/core/lib/debug/stats_data.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/experiments/experiments.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/match.h"
 #include "src/core/lib/gprpp/ref_counted.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
@@ -73,6 +68,10 @@
 #include "src/core/lib/transport/http2_errors.h"
 #include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/transport.h"
+#include "src/core/telemetry/call_tracer.h"
+#include "src/core/telemetry/stats.h"
+#include "src/core/telemetry/stats_data.h"
+#include "src/core/util/useful.h"
 
 // IWYU pragma: no_include "src/core/lib/gprpp/orphanable.h"
 
@@ -133,44 +132,43 @@ static void maybe_initiate_ping(grpc_chttp2_transport* t) {
           t->channelz_socket->RecordKeepaliveSent();
         }
         grpc_core::global_stats().IncrementHttp2PingsSent();
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_http_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_bdp_estimator_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_keepalive_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_ping_trace)) {
-          gpr_log(GPR_INFO, "%s[%p]: Ping %" PRIx64 " sent [%s]: %s",
-                  t->is_client ? "CLIENT" : "SERVER", t, id,
-                  std::string(t->peer_string.as_string_view()).c_str(),
-                  t->ping_rate_policy.GetDebugString().c_str());
+        if (GRPC_TRACE_FLAG_ENABLED(http) ||
+            GRPC_TRACE_FLAG_ENABLED(bdp_estimator) ||
+            GRPC_TRACE_FLAG_ENABLED(http_keepalive) ||
+            GRPC_TRACE_FLAG_ENABLED(http2_ping)) {
+          LOG(INFO) << (t->is_client ? "CLIENT" : "SERVER") << "[" << t
+                    << "]: Ping " << id << " sent ["
+                    << std::string(t->peer_string.as_string_view())
+                    << "]: " << t->ping_rate_policy.GetDebugString();
         }
       },
       [t](grpc_core::Chttp2PingRatePolicy::TooManyRecentPings) {
         // need to receive something of substance before sending a ping again
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_http_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_bdp_estimator_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_keepalive_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_ping_trace)) {
-          gpr_log(GPR_INFO,
-                  "%s[%p]: Ping delayed [%s]: too many recent pings: %s",
-                  t->is_client ? "CLIENT" : "SERVER", t,
-                  std::string(t->peer_string.as_string_view()).c_str(),
-                  t->ping_rate_policy.GetDebugString().c_str());
+        if (GRPC_TRACE_FLAG_ENABLED(http) ||
+            GRPC_TRACE_FLAG_ENABLED(bdp_estimator) ||
+            GRPC_TRACE_FLAG_ENABLED(http_keepalive) ||
+            GRPC_TRACE_FLAG_ENABLED(http2_ping)) {
+          LOG(INFO) << (t->is_client ? "CLIENT" : "SERVER") << "[" << t
+                    << "]: Ping delayed ["
+                    << std::string(t->peer_string.as_string_view())
+                    << "]: too many recent pings: "
+                    << t->ping_rate_policy.GetDebugString();
         }
       },
       [t](grpc_core::Chttp2PingRatePolicy::TooSoon too_soon) {
         // not enough elapsed time between successive pings
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_http_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_bdp_estimator_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_keepalive_trace) ||
-            GRPC_TRACE_FLAG_ENABLED(grpc_ping_trace)) {
-          gpr_log(
-              GPR_INFO,
-              "%s[%p]: Ping delayed [%s]: not enough time elapsed since last "
-              "ping. Last ping:%s, minimum wait:%s need to wait:%s",
-              t->is_client ? "CLIENT" : "SERVER", t,
-              std::string(t->peer_string.as_string_view()).c_str(),
-              too_soon.last_ping.ToString().c_str(),
-              too_soon.next_allowed_ping_interval.ToString().c_str(),
-              too_soon.wait.ToString().c_str());
+        if (GRPC_TRACE_FLAG_ENABLED(http) ||
+            GRPC_TRACE_FLAG_ENABLED(bdp_estimator) ||
+            GRPC_TRACE_FLAG_ENABLED(http_keepalive) ||
+            GRPC_TRACE_FLAG_ENABLED(http2_ping)) {
+          LOG(INFO) << (t->is_client ? "CLIENT" : "SERVER") << "[" << t
+                    << "]: Ping delayed ["
+                    << std::string(t->peer_string.as_string_view())
+                    << "]: not enough time elapsed since last "
+                       "ping. Last ping:"
+                    << too_soon.last_ping
+                    << ", minimum wait:" << too_soon.next_allowed_ping_interval
+                    << ", need to wait:" << too_soon.wait;
         }
         if (t->delayed_ping_timer_handle ==
             grpc_event_engine::experimental::EventEngine::TaskHandle::
@@ -207,23 +205,23 @@ static bool update_list(grpc_chttp2_transport* t, int64_t send_bytes,
 
 static void report_stall(grpc_chttp2_transport* t, grpc_chttp2_stream* s,
                          const char* staller) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_flowctl_trace)) {
-    gpr_log(
-        GPR_DEBUG,
-        "%s:%p stream %d moved to stalled list by %s. This is FULLY expected "
-        "to happen in a healthy program that is not seeing flow control stalls."
-        " However, if you know that there are unwanted stalls, here is some "
-        "helpful data: [fc:pending=%" PRIdPTR ":flowed=%" PRId64
-        ":peer_initwin=%d:t_win=%" PRId64 ":s_win=%d:s_delta=%" PRId64 "]",
-        std::string(t->peer_string.as_string_view()).c_str(), t, s->id, staller,
-        s->flow_controlled_buffer.length, s->flow_controlled_bytes_flowed,
-        t->settings.acked().initial_window_size(),
-        t->flow_control.remote_window(),
-        static_cast<uint32_t>(std::max(
-            int64_t{0}, s->flow_control.remote_window_delta() +
-                            static_cast<int64_t>(
-                                t->settings.peer().initial_window_size()))),
-        s->flow_control.remote_window_delta());
+  if (GRPC_TRACE_FLAG_ENABLED(flowctl)) {
+    VLOG(2) << t->peer_string.as_string_view() << ":" << t << " stream "
+            << s->id << " moved to stalled list by " << staller
+            << ". This is FULLY expected to happen in a healthy program that "
+               "is not seeing flow control stalls. However, if you know that "
+               "there are unwanted stalls, here is some helpful data: "
+               "[fc:pending="
+            << s->flow_controlled_buffer.length
+            << ":flowed=" << s->flow_controlled_bytes_flowed
+            << ":peer_initwin=" << t->settings.acked().initial_window_size()
+            << ":t_win=" << t->flow_control.remote_window() << ":s_win="
+            << static_cast<uint32_t>(
+                   std::max(int64_t{0},
+                            s->flow_control.remote_window_delta() +
+                                static_cast<int64_t>(
+                                    t->settings.peer().initial_window_size())))
+            << ":s_delta=" << s->flow_control.remote_window_delta() << "]";
   }
 }
 
@@ -298,10 +296,9 @@ class WriteContext {
     uint32_t transport_announce = t_->flow_control.MaybeSendUpdate(
         t_->outbuf.c_slice_buffer()->count > 0);
     if (transport_announce) {
-      grpc_transport_one_way_stats throwaway_stats;
-      grpc_slice_buffer_add(t_->outbuf.c_slice_buffer(),
-                            grpc_chttp2_window_update_create(
-                                0, transport_announce, &throwaway_stats));
+      grpc_slice_buffer_add(
+          t_->outbuf.c_slice_buffer(),
+          grpc_chttp2_window_update_create(0, transport_announce, nullptr));
       grpc_chttp2_reset_ping_clock(t_);
     }
   }
@@ -413,7 +410,7 @@ class DataSendContext {
                      s_->send_trailing_metadata != nullptr &&
                      s_->send_trailing_metadata->empty();
     grpc_chttp2_encode_data(s_->id, &s_->flow_controlled_buffer, send_bytes,
-                            is_last_frame_, &s_->stats.outgoing,
+                            is_last_frame_, &s_->call_tracer_wrapper,
                             t_->outbuf.c_slice_buffer());
     sfc_upd_.SentData(send_bytes);
     s_->sending_bytes += send_bytes;
@@ -444,10 +441,10 @@ class StreamWriteContext {
  public:
   StreamWriteContext(WriteContext* write_context, grpc_chttp2_stream* s)
       : write_context_(write_context), t_(write_context->transport()), s_(s) {
-    GRPC_CHTTP2_IF_TRACING(
-        gpr_log(GPR_INFO, "W:%p %s[%d] im-(sent,send)=(%d,%d)", t_,
-                t_->is_client ? "CLIENT" : "SERVER", s->id,
-                s->sent_initial_metadata, s->send_initial_metadata != nullptr));
+    GRPC_CHTTP2_IF_TRACING(INFO)
+        << "W:" << t_ << " " << (t_->is_client ? "CLIENT" : "SERVER") << "["
+        << s->id << "] im-(sent,send)=(" << s->sent_initial_metadata << ","
+        << (s->send_initial_metadata != nullptr) << ")";
   }
 
   void FlushInitialMetadata() {
@@ -472,8 +469,7 @@ class StreamWriteContext {
               t_->settings.peer()
                   .allow_true_binary_metadata(),     // use_true_binary_metadata
               t_->settings.peer().max_frame_size(),  // max_frame_size
-              &s_->stats.outgoing                    // stats
-          },
+              &s_->call_tracer_wrapper},
           *s_->send_initial_metadata, t_->outbuf.c_slice_buffer());
       grpc_chttp2_reset_ping_clock(t_);
       write_context_->IncInitialMetadataWrites();
@@ -485,16 +481,32 @@ class StreamWriteContext {
     grpc_chttp2_complete_closure_step(t_, &s_->send_initial_metadata_finished,
                                       absl::OkStatus(),
                                       "send_initial_metadata_finished");
-    if (s_->call_tracer) {
-      grpc_core::HttpAnnotation::WriteStats write_stats;
-      write_stats.target_write_size = write_context_->target_write_size();
-      s_->call_tracer->RecordAnnotation(
-          grpc_core::HttpAnnotation(
-              grpc_core::HttpAnnotation::Type::kHeadWritten,
-              gpr_now(GPR_CLOCK_REALTIME))
-              .Add(s_->t->flow_control.stats())
-              .Add(s_->flow_control.stats())
-              .Add(write_stats));
+    if (!grpc_core::IsCallTracerInTransportEnabled()) {
+      if (s_->call_tracer) {
+        grpc_core::HttpAnnotation::WriteStats write_stats;
+        write_stats.target_write_size = write_context_->target_write_size();
+        s_->call_tracer->RecordAnnotation(
+            grpc_core::HttpAnnotation(
+                grpc_core::HttpAnnotation::Type::kHeadWritten,
+                gpr_now(GPR_CLOCK_REALTIME))
+                .Add(s_->t->flow_control.stats())
+                .Add(s_->flow_control.stats())
+                .Add(write_stats));
+      }
+    } else if (grpc_core::IsTraceRecordCallopsEnabled()) {
+      auto* call_tracer =
+          s_->arena->GetContext<grpc_core::CallTracerInterface>();
+      if (call_tracer != nullptr && call_tracer->IsSampled()) {
+        grpc_core::HttpAnnotation::WriteStats write_stats;
+        write_stats.target_write_size = write_context_->target_write_size();
+        call_tracer->RecordAnnotation(
+            grpc_core::HttpAnnotation(
+                grpc_core::HttpAnnotation::Type::kHeadWritten,
+                gpr_now(GPR_CLOCK_REALTIME))
+                .Add(s_->t->flow_control.stats())
+                .Add(s_->flow_control.stats())
+                .Add(write_stats));
+      }
     }
   }
 
@@ -505,9 +517,10 @@ class StreamWriteContext {
     const uint32_t stream_announce = s_->flow_control.MaybeSendUpdate();
     if (stream_announce == 0) return;
 
-    grpc_slice_buffer_add(t_->outbuf.c_slice_buffer(),
-                          grpc_chttp2_window_update_create(
-                              s_->id, stream_announce, &s_->stats.outgoing));
+    grpc_slice_buffer_add(
+        t_->outbuf.c_slice_buffer(),
+        grpc_chttp2_window_update_create(s_->id, stream_announce,
+                                         &s_->call_tracer_wrapper));
     grpc_chttp2_reset_ping_clock(t_);
     write_context_->IncWindowUpdateWrites();
   }
@@ -557,15 +570,16 @@ class StreamWriteContext {
     if (s_->send_trailing_metadata == nullptr) return;
     if (s_->flow_controlled_buffer.length != 0) return;
 
-    GRPC_CHTTP2_IF_TRACING(gpr_log(GPR_INFO, "sending trailing_metadata"));
+    GRPC_CHTTP2_IF_TRACING(INFO) << "sending trailing_metadata";
     if (s_->send_trailing_metadata->empty()) {
       grpc_chttp2_encode_data(s_->id, &s_->flow_controlled_buffer, 0, true,
-                              &s_->stats.outgoing, t_->outbuf.c_slice_buffer());
+                              &s_->call_tracer_wrapper,
+                              t_->outbuf.c_slice_buffer());
     } else {
       t_->hpack_compressor.EncodeHeaders(
           grpc_core::HPackCompressor::EncodeHeaderOptions{
               s_->id, true, t_->settings.peer().allow_true_binary_metadata(),
-              t_->settings.peer().max_frame_size(), &s_->stats.outgoing},
+              t_->settings.peer().max_frame_size(), &s_->call_tracer_wrapper},
           *s_->send_trailing_metadata, t_->outbuf.c_slice_buffer());
     }
     write_context_->IncTrailingMetadataWrites();
@@ -608,8 +622,8 @@ class StreamWriteContext {
   };
 
   void ConvertInitialMetadataToTrailingMetadata() {
-    GRPC_CHTTP2_IF_TRACING(
-        gpr_log(GPR_INFO, "not sending initial_metadata (Trailers-Only)"));
+    GRPC_CHTTP2_IF_TRACING(INFO)
+        << "not sending initial_metadata (Trailers-Only)";
     // When sending Trailers-Only, we need to move metadata from headers to
     // trailers.
     TrailersOnlyMetadataEncoder encoder(s_->send_trailing_metadata);
@@ -629,16 +643,28 @@ class StreamWriteContext {
       grpc_slice_buffer_add(
           t_->outbuf.c_slice_buffer(),
           grpc_chttp2_rst_stream_create(s_->id, GRPC_HTTP2_NO_ERROR,
-                                        &s_->stats.outgoing));
+                                        &s_->call_tracer_wrapper));
     }
     grpc_chttp2_mark_stream_closed(t_, s_, !t_->is_client, true,
                                    absl::OkStatus());
-    if (s_->call_tracer) {
-      s_->call_tracer->RecordAnnotation(
-          grpc_core::HttpAnnotation(grpc_core::HttpAnnotation::Type::kEnd,
-                                    gpr_now(GPR_CLOCK_REALTIME))
-              .Add(s_->t->flow_control.stats())
-              .Add(s_->flow_control.stats()));
+    if (!grpc_core::IsCallTracerInTransportEnabled()) {
+      if (s_->call_tracer) {
+        s_->call_tracer->RecordAnnotation(
+            grpc_core::HttpAnnotation(grpc_core::HttpAnnotation::Type::kEnd,
+                                      gpr_now(GPR_CLOCK_REALTIME))
+                .Add(s_->t->flow_control.stats())
+                .Add(s_->flow_control.stats()));
+      }
+    } else if (grpc_core::IsTraceRecordCallopsEnabled()) {
+      auto* call_tracer =
+          s_->arena->GetContext<grpc_core::CallTracerInterface>();
+      if (call_tracer != nullptr && call_tracer->IsSampled()) {
+        call_tracer->RecordAnnotation(
+            grpc_core::HttpAnnotation(grpc_core::HttpAnnotation::Type::kEnd,
+                                      gpr_now(GPR_CLOCK_REALTIME))
+                .Add(s_->t->flow_control.stats())
+                .Add(s_->flow_control.stats()));
+      }
     }
   }
 
@@ -677,15 +703,15 @@ grpc_chttp2_begin_write_result grpc_chttp2_begin_write(
       num_stream_bytes = t->outbuf.c_slice_buffer()->length - orig_len;
       s->byte_counter += static_cast<size_t>(num_stream_bytes);
       ++s->write_counter;
-      if (s->traced && grpc_endpoint_can_track_err(t->ep)) {
+      if (s->traced && grpc_endpoint_can_track_err(t->ep.get())) {
         grpc_core::CopyContextFn copy_context_fn =
             grpc_core::GrpcHttp2GetCopyContextFn();
         if (copy_context_fn != nullptr &&
             grpc_core::GrpcHttp2GetWriteTimestampsCallback() != nullptr) {
-          t->cl->emplace_back(copy_context_fn(s->context),
-                              outbuf_relative_start_pos, num_stream_bytes,
-                              s->byte_counter, s->write_counter - 1,
-                              s->tcp_tracer);
+          t->context_list->emplace_back(copy_context_fn(s->arena),
+                                        outbuf_relative_start_pos,
+                                        num_stream_bytes, s->byte_counter,
+                                        s->write_counter - 1, s->tcp_tracer);
         }
       }
       outbuf_relative_start_pos += num_stream_bytes;
@@ -728,11 +754,10 @@ void grpc_chttp2_end_write(grpc_chttp2_transport* t, grpc_error_handle error) {
           grpc_core::ExecCtx exec_ctx;
           grpc_chttp2_ping_timeout(t);
         });
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_ping_trace) && id.has_value()) {
-      gpr_log(GPR_INFO,
-              "%s[%p]: Set ping timeout timer of %s for ping id %" PRIx64,
-              t->is_client ? "CLIENT" : "SERVER", t, timeout.ToString().c_str(),
-              id.value());
+    if (GRPC_TRACE_FLAG_ENABLED(http2_ping) && id.has_value()) {
+      LOG(INFO) << (t->is_client ? "CLIENT" : "SERVER") << "[" << t
+                << "]: Set ping timeout timer of " << timeout.ToString()
+                << " for ping id " << id.value();
     }
 
     if (t->keepalive_incoming_data_wanted &&
@@ -740,11 +765,11 @@ void grpc_chttp2_end_write(grpc_chttp2_transport* t, grpc_error_handle error) {
         t->keepalive_ping_timeout_handle !=
             grpc_event_engine::experimental::EventEngine::TaskHandle::
                 kInvalid) {
-      if (GRPC_TRACE_FLAG_ENABLED(grpc_ping_trace) ||
-          GRPC_TRACE_FLAG_ENABLED(grpc_keepalive_trace)) {
-        gpr_log(GPR_INFO, "%s[%p]: Set keepalive ping timeout timer of %s",
-                t->is_client ? "CLIENT" : "SERVER", t,
-                t->keepalive_timeout.ToString().c_str());
+      if (GRPC_TRACE_FLAG_ENABLED(http2_ping) ||
+          GRPC_TRACE_FLAG_ENABLED(http_keepalive)) {
+        LOG(INFO) << (t->is_client ? "CLIENT" : "SERVER") << "[" << t
+                  << "]: Set keepalive ping timeout timer of "
+                  << t->keepalive_timeout.ToString();
       }
       t->keepalive_ping_timeout_handle =
           t->event_engine->RunAfter(t->keepalive_timeout, [t = t->Ref()] {
