@@ -26,17 +26,17 @@
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/debug/trace.h"
+#include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/iomgr/endpoint.h"
 
-extern grpc_core::TraceFlag grpc_trace_secure_endpoint;
-
 // Takes ownership of protector, zero_copy_protector, and to_wrap, and refs
 // leftover_slices. If zero_copy_protector is not NULL, protector will never be
 // used.
-grpc_endpoint* grpc_secure_endpoint_create(
+grpc_core::OrphanablePtr<grpc_endpoint> grpc_secure_endpoint_create(
     struct tsi_frame_protector* protector,
     struct tsi_zero_copy_grpc_protector* zero_copy_protector,
-    grpc_endpoint* to_wrap, grpc_slice* leftover_slices,
-    const grpc_channel_args* channel_args, size_t leftover_nslices);
+    grpc_core::OrphanablePtr<grpc_endpoint> to_wrap,
+    grpc_slice* leftover_slices, const grpc_channel_args* channel_args,
+    size_t leftover_nslices);
 
 #endif  // GRPC_SRC_CORE_HANDSHAKER_SECURITY_SECURE_ENDPOINT_H
