@@ -19,6 +19,7 @@
 
 #include "absl/functional/any_invocable.h"
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
@@ -27,17 +28,14 @@
 #include <grpc/impl/codegen/slice.h>
 #include <grpc/slice.h>
 #include <grpc/slice_buffer.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
+#include "src/core/lib/debug/trace.h"
 #include "src/core/lib/event_engine/extensions/can_track_errors.h"
 #include "src/core/lib/event_engine/extensions/supports_fd.h"
 #include "src/core/lib/event_engine/query_extensions.h"
-#include "src/core/lib/event_engine/shim.h"
 #include "src/core/lib/event_engine/tcp_socket_utils.h"
-#include "src/core/lib/event_engine/trace.h"
-#include "src/core/lib/gpr/string.h"
 #include "src/core/lib/gprpp/construct_destruct.h"
 #include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/gprpp/sync.h"
@@ -49,8 +47,7 @@
 #include "src/core/lib/iomgr/port.h"
 #include "src/core/lib/slice/slice_string_helpers.h"
 #include "src/core/lib/transport/error_utils.h"
-
-extern grpc_core::TraceFlag grpc_tcp_trace;
+#include "src/core/util/string.h"
 
 namespace grpc_event_engine {
 namespace experimental {
@@ -120,15 +117,14 @@ class EventEngineEndpointWrapper {
     grpc_slice_buffer_move_into(read_buffer->c_slice_buffer(),
                                 pending_read_buffer_);
     read_buffer->~SliceBuffer();
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
+    if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
       size_t i;
-      gpr_log(GPR_INFO, "TCP: %p READ error=%s", eeep_->wrapper,
-              status.ToString().c_str());
-      if (gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+      LOG(INFO) << "TCP: " << eeep_->wrapper << " READ error=" << status;
+      if (ABSL_VLOG_IS_ON(2)) {
         for (i = 0; i < pending_read_buffer_->count; i++) {
           char* dump = grpc_dump_slice(pending_read_buffer_->slices[i],
                                        GPR_DUMP_HEX | GPR_DUMP_ASCII);
-          gpr_log(GPR_DEBUG, "READ DATA: %s", dump);
+          VLOG(2) << "READ DATA: " << dump;
           gpr_free(dump);
         }
       }
@@ -151,15 +147,14 @@ class EventEngineEndpointWrapper {
   bool Write(grpc_closure* write_cb, grpc_slice_buffer* slices,
              const EventEngine::Endpoint::WriteArgs* args) {
     Ref();
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
+    if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
       size_t i;
-      gpr_log(GPR_INFO, "TCP: %p WRITE (peer=%s)", this,
-              std::string(PeerAddress()).c_str());
-      if (gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+      LOG(INFO) << "TCP: " << this << " WRITE (peer=" << PeerAddress() << ")";
+      if (ABSL_VLOG_IS_ON(2)) {
         for (i = 0; i < slices->count; i++) {
           char* dump =
               grpc_dump_slice(slices->slices[i], GPR_DUMP_HEX | GPR_DUMP_ASCII);
-          gpr_log(GPR_DEBUG, "WRITE DATA: %s", dump);
+          VLOG(2) << "WRITE DATA: " << dump;
           gpr_free(dump);
         }
       }
@@ -178,9 +173,9 @@ class EventEngineEndpointWrapper {
   void FinishPendingWrite(absl::Status status) {
     auto* write_buffer = reinterpret_cast<SliceBuffer*>(&eeep_->write_buffer);
     write_buffer->~SliceBuffer();
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
-      gpr_log(GPR_INFO, "TCP: %p WRITE (peer=%s) error=%s", this,
-              std::string(PeerAddress()).c_str(), status.ToString().c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
+      LOG(INFO) << "TCP: " << this << " WRITE (peer=" << PeerAddress()
+                << ") error=" << status;
     }
     grpc_closure* cb = pending_write_cb_;
     pending_write_cb_ = nullptr;
@@ -345,29 +340,18 @@ void EndpointAddToPollsetSet(grpc_endpoint* /* ep */,
                              grpc_pollset_set* /* pollset */) {}
 void EndpointDeleteFromPollsetSet(grpc_endpoint* /* ep */,
                                   grpc_pollset_set* /* pollset */) {}
-/// After shutdown, all endpoint operations except destroy are no-op,
-/// and will return some kind of sane default (empty strings, nullptrs, etc).
-/// It is the caller's responsibility to ensure that calls to EndpointShutdown
-/// are synchronized.
-void EndpointShutdown(grpc_endpoint* ep, grpc_error_handle why) {
-  auto* eeep =
-      reinterpret_cast<EventEngineEndpointWrapper::grpc_event_engine_endpoint*>(
-          ep);
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
-    gpr_log(GPR_INFO, "TCP Endpoint %p shutdown why=%s", eeep->wrapper,
-            why.ToString().c_str());
-  }
-  GRPC_EVENT_ENGINE_TRACE("EventEngine::Endpoint %p Shutdown:%s", eeep->wrapper,
-                          why.ToString().c_str());
-  eeep->wrapper->TriggerShutdown(nullptr);
-}
 
-// Attempts to free the underlying data structures.
+/// Attempts to free the underlying data structures.
+/// After destruction, no new endpoint operations may be started.
+/// It is the caller's responsibility to ensure that calls to EndpointDestroy
+/// are synchronized.
 void EndpointDestroy(grpc_endpoint* ep) {
   auto* eeep =
       reinterpret_cast<EventEngineEndpointWrapper::grpc_event_engine_endpoint*>(
           ep);
-  GRPC_EVENT_ENGINE_TRACE("EventEngine::Endpoint %p Destroy", eeep->wrapper);
+  GRPC_TRACE_LOG(event_engine, INFO)
+      << "EventEngine::Endpoint::" << eeep->wrapper << " EndpointDestroy";
+  eeep->wrapper->TriggerShutdown(nullptr);
   eeep->wrapper->Unref();
 }
 
@@ -405,7 +389,6 @@ grpc_endpoint_vtable grpc_event_engine_endpoint_vtable = {
     EndpointAddToPollset,
     EndpointAddToPollsetSet,
     EndpointDeleteFromPollsetSet,
-    EndpointShutdown,
     EndpointDestroy,
     EndpointGetPeerAddress,
     EndpointGetLocalAddress,
@@ -425,7 +408,8 @@ EventEngineEndpointWrapper::EventEngineEndpointWrapper(
   } else {
     fd_ = -1;
   }
-  GRPC_EVENT_ENGINE_TRACE("EventEngine::Endpoint %p Create", eeep_->wrapper);
+  GRPC_TRACE_LOG(event_engine, INFO)
+      << "EventEngine::Endpoint " << eeep_->wrapper << " Create";
 }
 
 }  // namespace
