@@ -25,16 +25,13 @@
 #include <limits.h>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 
 #include <grpc/slice_buffer.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
-#include <grpc/support/log_windows.h>
 #include <grpc/support/string_util.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
-#include "src/core/lib/gpr/string.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/iomgr/iocp_windows.h"
 #include "src/core/lib/iomgr/sockaddr.h"
@@ -45,6 +42,8 @@
 #include "src/core/lib/iomgr/timer.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/lib/slice/slice_string_helpers.h"
+#include "src/core/util/string.h"
+#include "src/core/util/useful.h"
 
 #if defined(__MSYS__) && defined(GPR_ARCH_64)
 // Nasty workaround for nasty bug when using the 64 bits msys compiler
@@ -54,8 +53,6 @@
 #define GRPC_FIONBIO FIONBIO
 #endif
 
-extern grpc_core::TraceFlag grpc_tcp_trace;
-
 grpc_error_handle grpc_tcp_set_non_block(SOCKET sock) {
   int status;
   uint32_t param = 1;
@@ -124,7 +121,6 @@ typedef struct grpc_tcp {
   // to protect ourselves when requesting a shutdown.
   gpr_mu mu;
   int shutting_down;
-  grpc_error_handle shutdown_error;
 
   std::string peer_string;
   std::string local_address;
@@ -142,11 +138,10 @@ static void tcp_free(grpc_tcp* tcp) {
 #define TCP_REF(tcp, reason) tcp_ref((tcp), (reason), __FILE__, __LINE__)
 static void tcp_unref(grpc_tcp* tcp, const char* reason, const char* file,
                       int line) {
-  if (grpc_tcp_trace.enabled()) {
+  if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
     gpr_atm val = gpr_atm_no_barrier_load(&tcp->refcount.count);
-    gpr_log(file, line, GPR_LOG_SEVERITY_DEBUG,
-            "TCP unref %p : %s %" PRIdPTR " -> %" PRIdPTR, tcp, reason, val,
-            val - 1);
+    VLOG(2).AtLocation(file, line) << "TCP unref " << tcp << " : " << reason
+                                   << " " << val << " -> " << val - 1;
   }
   if (gpr_unref(&tcp->refcount)) {
     tcp_free(tcp);
@@ -155,11 +150,10 @@ static void tcp_unref(grpc_tcp* tcp, const char* reason, const char* file,
 
 static void tcp_ref(grpc_tcp* tcp, const char* reason, const char* file,
                     int line) {
-  if (grpc_tcp_trace.enabled()) {
+  if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
     gpr_atm val = gpr_atm_no_barrier_load(&tcp->refcount.count);
-    gpr_log(file, line, GPR_LOG_SEVERITY_DEBUG,
-            "TCP   ref %p : %s %" PRIdPTR " -> %" PRIdPTR, tcp, reason, val,
-            val + 1);
+    VLOG(2).AtLocation(file, line) << "TCP   ref " << tcp << " : " << reason
+                                   << " " << val << " -> " << val + 1;
   }
   gpr_ref(&tcp->refcount);
 }
@@ -182,8 +176,8 @@ static void on_read(void* tcpp, grpc_error_handle error) {
   grpc_winsocket* socket = tcp->socket;
   grpc_winsocket_callback_info* info = &socket->read_info;
 
-  if (grpc_tcp_trace.enabled()) {
-    gpr_log(GPR_INFO, "TCP:%p on_read", tcp);
+  if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
+    LOG(INFO) << "TCP:" << tcp << " on_read";
   }
 
   if (error.ok()) {
@@ -203,27 +197,24 @@ static void on_read(void* tcpp, grpc_error_handle error) {
         }
         CHECK((size_t)info->bytes_transferred == tcp->read_slices->length);
 
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace) &&
-            gpr_should_log(GPR_LOG_SEVERITY_INFO)) {
+        if (GRPC_TRACE_FLAG_ENABLED(tcp) && ABSL_VLOG_IS_ON(2)) {
           size_t i;
           for (i = 0; i < tcp->read_slices->count; i++) {
             char* dump = grpc_dump_slice(tcp->read_slices->slices[i],
                                          GPR_DUMP_HEX | GPR_DUMP_ASCII);
-            gpr_log(GPR_INFO, "READ %p (peer=%s): %s", tcp,
-                    tcp->peer_string.c_str(), dump);
+            VLOG(2) << "READ " << tcp << " (peer=" << tcp->peer_string
+                    << "): " << dump;
             gpr_free(dump);
           }
         }
       } else {
-        if (grpc_tcp_trace.enabled()) {
-          gpr_log(GPR_INFO, "TCP:%p unref read_slice", tcp);
+        if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
+          LOG(INFO) << "TCP:" << tcp << " unref read_slice";
         }
         grpc_slice_buffer_reset_and_unref(tcp->read_slices);
         error = grpc_error_set_int(
-            tcp->shutting_down
-                ? GRPC_ERROR_CREATE_REFERENCING("TCP stream shutting down",
-                                                &tcp->shutdown_error, 1)
-                : GRPC_ERROR_CREATE("End of TCP stream"),
+            tcp->shutting_down ? GRPC_ERROR_CREATE("TCP stream shutting down")
+                               : GRPC_ERROR_CREATE("End of TCP stream"),
             grpc_core::StatusIntProperty::kRpcStatus, GRPC_STATUS_UNAVAILABLE);
       }
     }
@@ -248,17 +239,16 @@ static void win_read(grpc_endpoint* ep, grpc_slice_buffer* read_slices,
   WSABUF buffers[MAX_WSABUF_COUNT];
   size_t i;
 
-  if (grpc_tcp_trace.enabled()) {
-    gpr_log(GPR_INFO, "TCP:%p win_read", tcp);
+  if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
+    LOG(INFO) << "TCP:" << tcp << " win_read";
   }
 
   if (tcp->shutting_down) {
     grpc_core::ExecCtx::Run(
         DEBUG_LOCATION, cb,
-        grpc_error_set_int(
-            GRPC_ERROR_CREATE_REFERENCING("TCP socket is shutting down",
-                                          &tcp->shutdown_error, 1),
-            grpc_core::StatusIntProperty::kRpcStatus, GRPC_STATUS_UNAVAILABLE));
+        grpc_error_set_int(GRPC_ERROR_CREATE("TCP socket is shutting down"),
+                           grpc_core::StatusIntProperty::kRpcStatus,
+                           GRPC_STATUS_UNAVAILABLE));
     return;
   }
 
@@ -320,8 +310,8 @@ static void on_write(void* tcpp, grpc_error_handle error) {
   grpc_winsocket_callback_info* info = &handle->write_info;
   grpc_closure* cb;
 
-  if (grpc_tcp_trace.enabled()) {
-    gpr_log(GPR_INFO, "TCP:%p on_write", tcp);
+  if (GRPC_TRACE_FLAG_ENABLED(tcp)) {
+    LOG(INFO) << "TCP:" << tcp << " on_write";
   }
 
   gpr_mu_lock(&tcp->mu);
@@ -356,14 +346,13 @@ static void win_write(grpc_endpoint* ep, grpc_slice_buffer* slices,
   WSABUF* buffers = local_buffers;
   size_t len, async_buffers_offset = 0;
 
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace) &&
-      gpr_should_log(GPR_LOG_SEVERITY_INFO)) {
+  if (GRPC_TRACE_FLAG_ENABLED(tcp) && ABSL_VLOG_IS_ON(2)) {
     size_t i;
     for (i = 0; i < slices->count; i++) {
       char* data =
           grpc_dump_slice(slices->slices[i], GPR_DUMP_HEX | GPR_DUMP_ASCII);
-      gpr_log(GPR_INFO, "WRITE %p (peer=%s): %s", tcp, tcp->peer_string.c_str(),
-              data);
+      VLOG(2) << "WRITE " << tcp << " (peer=" << tcp->peer_string
+              << "): " << data;
       gpr_free(data);
     }
   }
@@ -371,10 +360,9 @@ static void win_write(grpc_endpoint* ep, grpc_slice_buffer* slices,
   if (tcp->shutting_down) {
     grpc_core::ExecCtx::Run(
         DEBUG_LOCATION, cb,
-        grpc_error_set_int(
-            GRPC_ERROR_CREATE_REFERENCING("TCP socket is shutting down",
-                                          &tcp->shutdown_error, 1),
-            grpc_core::StatusIntProperty::kRpcStatus, GRPC_STATUS_UNAVAILABLE));
+        grpc_error_set_int(GRPC_ERROR_CREATE("TCP socket is shutting down"),
+                           grpc_core::StatusIntProperty::kRpcStatus,
+                           GRPC_STATUS_UNAVAILABLE));
     return;
   }
 
@@ -479,21 +467,14 @@ static void win_delete_from_pollset_set(grpc_endpoint* /* ep */,
 // we're not going to protect against these. However the IO Completion Port
 // callback will happen from another thread, so we need to protect against
 // concurrent access of the data structure in that regard.
-static void win_shutdown(grpc_endpoint* ep, grpc_error_handle why) {
+static void win_destroy(grpc_endpoint* ep) {
   grpc_tcp* tcp = (grpc_tcp*)ep;
   gpr_mu_lock(&tcp->mu);
   // At that point, what may happen is that we're already inside the IOCP
   // callback. See the comments in on_read and on_write.
-  if (!tcp->shutting_down) {
-    tcp->shutting_down = 1;
-    tcp->shutdown_error = why;
-  }
+  tcp->shutting_down = 1;
   grpc_winsocket_shutdown(tcp->socket);
   gpr_mu_unlock(&tcp->mu);
-}
-
-static void win_destroy(grpc_endpoint* ep) {
-  grpc_tcp* tcp = (grpc_tcp*)ep;
   grpc_slice_buffer_reset_and_unref(&tcp->last_read_buffer);
   TCP_UNREF(tcp, "destroy");
 }
@@ -517,7 +498,6 @@ static grpc_endpoint_vtable vtable = {win_read,
                                       win_add_to_pollset,
                                       win_add_to_pollset_set,
                                       win_delete_from_pollset_set,
-                                      win_shutdown,
                                       win_destroy,
                                       win_get_peer,
                                       win_get_local_address,
