@@ -26,6 +26,7 @@
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 
@@ -34,15 +35,14 @@
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/mpscq.h"
 #include "src/core/lib/promise/exec_ctx_wakeup_scheduler.h"
 #include "src/core/lib/promise/loop.h"
 #include "src/core/lib/promise/map.h"
 #include "src/core/lib/promise/race.h"
 #include "src/core/lib/promise/seq.h"
-#include "src/core/lib/resource_quota/trace.h"
 #include "src/core/lib/slice/slice_refcount.h"
+#include "src/core/util/useful.h"
 
 namespace grpc_core {
 
@@ -253,9 +253,9 @@ GrpcMemoryAllocatorImpl::GrpcMemoryAllocatorImpl(
 }
 
 GrpcMemoryAllocatorImpl::~GrpcMemoryAllocatorImpl() {
-  CHECK(free_bytes_.load(std::memory_order_acquire) +
-            sizeof(GrpcMemoryAllocatorImpl) ==
-        taken_bytes_.load(std::memory_order_relaxed));
+  CHECK_EQ(free_bytes_.load(std::memory_order_acquire) +
+               sizeof(GrpcMemoryAllocatorImpl),
+           taken_bytes_.load(std::memory_order_relaxed));
   memory_quota_->Return(taken_bytes_.load(std::memory_order_relaxed));
 }
 
@@ -355,8 +355,8 @@ void GrpcMemoryAllocatorImpl::MaybeDonateBack() {
     if (free_bytes_.compare_exchange_weak(free, new_free,
                                           std::memory_order_acq_rel,
                                           std::memory_order_acquire)) {
-      if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-        gpr_log(GPR_INFO, "[%p] Early return %" PRIdPTR " bytes", this, ret);
+      if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+        LOG(INFO) << "[" << this << "] Early return " << ret << " bytes";
       }
       CHECK(taken_bytes_.fetch_sub(ret, std::memory_order_relaxed) >= ret);
       memory_quota_->Return(ret);
@@ -450,13 +450,12 @@ void BasicMemoryQuota::Start() {
       [self](
           std::tuple<const char*, RefCountedPtr<ReclaimerQueue::Handle>> arg) {
         auto reclaimer = std::move(std::get<1>(arg));
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
+        if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
           double free = std::max(intptr_t{0}, self->free_bytes_.load());
           size_t quota_size = self->quota_size_.load();
-          gpr_log(GPR_INFO,
-                  "RQ: %s perform %s reclamation. Available free bytes: %f, "
-                  "total quota_size: %zu",
-                  self->name_.c_str(), std::get<0>(arg), free, quota_size);
+          LOG(INFO) << "RQ: " << self->name_ << " perform " << std::get<0>(arg)
+                    << " reclamation. Available free bytes: " << free
+                    << ", total quota_size: " << quota_size;
         }
         // One of the reclaimer queues gave us a way to get back memory.
         // Call the reclaimer with a token that contains enough to wake us
@@ -533,13 +532,12 @@ void BasicMemoryQuota::FinishReclamation(uint64_t token, Waker waker) {
   if (reclamation_counter_.compare_exchange_strong(current, current + 1,
                                                    std::memory_order_relaxed,
                                                    std::memory_order_relaxed)) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
+    if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
       double free = std::max(intptr_t{0}, free_bytes_.load());
       size_t quota_size = quota_size_.load();
-      gpr_log(GPR_INFO,
-              "RQ: %s reclamation complete. Available free bytes: %f, "
-              "total quota_size: %zu",
-              name_.c_str(), free, quota_size);
+      LOG(INFO) << "RQ: " << name_
+                << " reclamation complete. Available free bytes: " << free
+                << ", total quota_size: " << quota_size;
     }
     waker.Wakeup();
   }
@@ -550,8 +548,8 @@ void BasicMemoryQuota::Return(size_t amount) {
 }
 
 void BasicMemoryQuota::AddNewAllocator(GrpcMemoryAllocatorImpl* allocator) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-    gpr_log(GPR_INFO, "Adding allocator %p", allocator);
+  if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+    LOG(INFO) << "Adding allocator " << allocator;
   }
 
   AllocatorBucket::Shard& shard = small_allocators_.SelectShard(allocator);
@@ -563,8 +561,8 @@ void BasicMemoryQuota::AddNewAllocator(GrpcMemoryAllocatorImpl* allocator) {
 }
 
 void BasicMemoryQuota::RemoveAllocator(GrpcMemoryAllocatorImpl* allocator) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-    gpr_log(GPR_INFO, "Removing allocator %p", allocator);
+  if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+    LOG(INFO) << "Removing allocator " << allocator;
   }
 
   AllocatorBucket::Shard& small_shard =
@@ -610,8 +608,8 @@ void BasicMemoryQuota::MaybeMoveAllocator(GrpcMemoryAllocatorImpl* allocator,
 
 void BasicMemoryQuota::MaybeMoveAllocatorBigToSmall(
     GrpcMemoryAllocatorImpl* allocator) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-    gpr_log(GPR_INFO, "Moving allocator %p to small", allocator);
+  if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+    LOG(INFO) << "Moving allocator " << allocator << " to small";
   }
 
   AllocatorBucket::Shard& old_shard = big_allocators_.SelectShard(allocator);
@@ -631,8 +629,8 @@ void BasicMemoryQuota::MaybeMoveAllocatorBigToSmall(
 
 void BasicMemoryQuota::MaybeMoveAllocatorSmallToBig(
     GrpcMemoryAllocatorImpl* allocator) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-    gpr_log(GPR_INFO, "Moving allocator %p to big", allocator);
+  if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+    LOG(INFO) << "Moving allocator " << allocator << " to big";
   }
 
   AllocatorBucket::Shard& old_shard = small_allocators_.SelectShard(allocator);
@@ -768,9 +766,9 @@ double PressureTracker::AddSampleAndGetControlValue(double sample) {
     } else {
       report = controller_.Update(current_estimate - kSetPoint);
     }
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_resource_quota_trace)) {
-      gpr_log(GPR_INFO, "RQ: pressure:%lf report:%lf controller:%s",
-              current_estimate, report, controller_.DebugString().c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(resource_quota)) {
+      LOG(INFO) << "RQ: pressure:" << current_estimate << " report:" << report
+                << " controller:" << controller_.DebugString();
     }
     report_.store(report, std::memory_order_relaxed);
   });
