@@ -24,18 +24,17 @@
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/types/optional.h"
 
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/string_util.h>
 
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/security/security_connector/ssl_utils.h"
-#include "src/core/lib/surface/api_trace.h"
 #include "src/core/tsi/ssl/session_cache/ssl_session_cache.h"
 #include "src/core/tsi/ssl_transport_security.h"
 #include "src/core/tsi/transport_security_interface.h"
@@ -54,7 +53,7 @@ grpc_ssl_credentials::grpc_ssl_credentials(
     const char* pem_root_certs =
         grpc_core::DefaultSslRootStore::GetPemRootCerts();
     if (pem_root_certs == nullptr) {
-      gpr_log(GPR_ERROR, "Could not get default pem root certs.");
+      LOG(ERROR) << "Could not get default pem root certs.";
     } else {
       char* default_roots = gpr_strdup(pem_root_certs);
       config_.pem_root_certs = default_roots;
@@ -85,9 +84,8 @@ grpc_ssl_credentials::create_security_connector(
     grpc_core::RefCountedPtr<grpc_call_credentials> call_creds,
     const char* target, grpc_core::ChannelArgs* args) {
   if (config_.pem_root_certs == nullptr) {
-    gpr_log(GPR_ERROR,
-            "No root certs in config. Client-side security connector must have "
-            "root certs.");
+    LOG(ERROR) << "No root certs in config. Client-side security connector "
+                  "must have root certs.";
     return nullptr;
   }
   absl::optional<std::string> overridden_target_name =
@@ -111,9 +109,7 @@ grpc_ssl_credentials::create_security_connector(
         &config_, config_.pem_root_certs, root_store_, session_cache,
         &factory_with_cache);
     if (status != GRPC_SECURITY_OK) {
-      gpr_log(GPR_ERROR,
-              "InitializeClientHandshakerFactory returned bad "
-              "status.");
+      LOG(ERROR) << "InitializeClientHandshakerFactory returned bad status.";
       return nullptr;
     }
     security_connector = grpc_ssl_channel_security_connector_create(
@@ -196,9 +192,8 @@ grpc_security_status grpc_ssl_credentials::InitializeClientHandshakerFactory(
                            config->pem_key_cert_pair->cert_chain != nullptr;
   tsi_ssl_client_handshaker_options options;
   if (pem_root_certs == nullptr) {
-    gpr_log(
-        GPR_ERROR,
-        "Handshaker factory creation failed. pem_root_certs cannot be nullptr");
+    LOG(ERROR) << "Handshaker factory creation failed. pem_root_certs cannot "
+                  "be nullptr";
     return GRPC_SECURITY_ERROR;
   }
   options.pem_root_certs = pem_root_certs;
@@ -217,8 +212,8 @@ grpc_security_status grpc_ssl_credentials::InitializeClientHandshakerFactory(
                                                             handshaker_factory);
   gpr_free(options.alpn_protocols);
   if (result != TSI_OK) {
-    gpr_log(GPR_ERROR, "Handshaker factory creation failed with %s.",
-            tsi_result_to_string(result));
+    LOG(ERROR) << "Handshaker factory creation failed with "
+               << tsi_result_to_string(result);
     return GRPC_SECURITY_ERROR;
   }
   return GRPC_SECURITY_OK;
@@ -229,12 +224,11 @@ grpc_security_status grpc_ssl_credentials::InitializeClientHandshakerFactory(
 grpc_channel_credentials* grpc_ssl_credentials_create(
     const char* pem_root_certs, grpc_ssl_pem_key_cert_pair* pem_key_cert_pair,
     const verify_peer_options* verify_options, void* reserved) {
-  GRPC_API_TRACE(
-      "grpc_ssl_credentials_create(pem_root_certs=%s, "
-      "pem_key_cert_pair=%p, "
-      "verify_options=%p, "
-      "reserved=%p)",
-      4, (pem_root_certs, pem_key_cert_pair, verify_options, reserved));
+  GRPC_TRACE_LOG(api, INFO)
+      << "grpc_ssl_credentials_create(pem_root_certs=" << pem_root_certs
+      << ", pem_key_cert_pair=" << pem_key_cert_pair
+      << ", verify_options=" << verify_options << ", reserved=" << reserved
+      << ")";
   CHECK_EQ(reserved, nullptr);
 
   return new grpc_ssl_credentials(
@@ -245,12 +239,11 @@ grpc_channel_credentials* grpc_ssl_credentials_create(
 grpc_channel_credentials* grpc_ssl_credentials_create_ex(
     const char* pem_root_certs, grpc_ssl_pem_key_cert_pair* pem_key_cert_pair,
     const grpc_ssl_verify_peer_options* verify_options, void* reserved) {
-  GRPC_API_TRACE(
-      "grpc_ssl_credentials_create(pem_root_certs=%s, "
-      "pem_key_cert_pair=%p, "
-      "verify_options=%p, "
-      "reserved=%p)",
-      4, (pem_root_certs, pem_key_cert_pair, verify_options, reserved));
+  GRPC_TRACE_LOG(api, INFO)
+      << "grpc_ssl_credentials_create(pem_root_certs=" << pem_root_certs
+      << ", pem_key_cert_pair=" << pem_key_cert_pair
+      << ", verify_options=" << verify_options << ", reserved=" << reserved
+      << ")";
   CHECK_EQ(reserved, nullptr);
 
   return new grpc_ssl_credentials(pem_root_certs, pem_key_cert_pair,
@@ -378,7 +371,7 @@ grpc_ssl_server_credentials_create_options_using_config(
     grpc_ssl_server_certificate_config* config) {
   grpc_ssl_server_credentials_options* options = nullptr;
   if (config == nullptr) {
-    gpr_log(GPR_ERROR, "Certificate config must not be NULL.");
+    LOG(ERROR) << "Certificate config must not be NULL.";
     goto done;
   }
   options = static_cast<grpc_ssl_server_credentials_options*>(
@@ -394,7 +387,7 @@ grpc_ssl_server_credentials_create_options_using_config_fetcher(
     grpc_ssl_client_certificate_request_type client_certificate_request,
     grpc_ssl_server_certificate_config_callback cb, void* user_data) {
   if (cb == nullptr) {
-    gpr_log(GPR_ERROR, "Invalid certificate config callback parameter.");
+    LOG(ERROR) << "Invalid certificate config callback parameter.";
     return nullptr;
   }
 
@@ -429,13 +422,12 @@ grpc_server_credentials* grpc_ssl_server_credentials_create_ex(
     size_t num_key_cert_pairs,
     grpc_ssl_client_certificate_request_type client_certificate_request,
     void* reserved) {
-  GRPC_API_TRACE(
-      "grpc_ssl_server_credentials_create_ex("
-      "pem_root_certs=%s, pem_key_cert_pairs=%p, num_key_cert_pairs=%lu, "
-      "client_certificate_request=%d, reserved=%p)",
-      5,
-      (pem_root_certs, pem_key_cert_pairs, (unsigned long)num_key_cert_pairs,
-       client_certificate_request, reserved));
+  GRPC_TRACE_LOG(api, INFO)
+      << "grpc_ssl_server_credentials_create_ex(pem_root_certs="
+      << pem_root_certs << ", pem_key_cert_pairs=" << pem_key_cert_pairs
+      << ", num_key_cert_pairs=" << (unsigned long)num_key_cert_pairs
+      << ", client_certificate_request=" << client_certificate_request
+      << ", reserved=" << reserved << ")";
   CHECK_EQ(reserved, nullptr);
 
   grpc_ssl_server_certificate_config* cert_config =
@@ -453,20 +445,18 @@ grpc_server_credentials* grpc_ssl_server_credentials_create_with_options(
   grpc_server_credentials* retval = nullptr;
 
   if (options == nullptr) {
-    gpr_log(GPR_ERROR,
-            "Invalid options trying to create SSL server credentials.");
+    LOG(ERROR) << "Invalid options trying to create SSL server credentials.";
     goto done;
   }
 
   if (options->certificate_config == nullptr &&
       options->certificate_config_fetcher == nullptr) {
-    gpr_log(GPR_ERROR,
-            "SSL server credentials options must specify either "
-            "certificate config or fetcher.");
+    LOG(ERROR) << "SSL server credentials options must specify either "
+                  "certificate config or fetcher.";
     goto done;
   } else if (options->certificate_config_fetcher != nullptr &&
              options->certificate_config_fetcher->cb == nullptr) {
-    gpr_log(GPR_ERROR, "Certificate config fetcher callback must not be NULL.");
+    LOG(ERROR) << "Certificate config fetcher callback must not be NULL.";
     goto done;
   }
 
