@@ -26,13 +26,13 @@
 
 #include "absl/functional/bind_front.h"
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 
 #include <grpc/grpc.h>
 #include <grpc/grpc_security_constants.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/string_util.h>
 
@@ -246,21 +246,18 @@ TlsChannelSecurityConnector::CreateTlsChannelSecurityConnector(
     const char* target_name, const char* overridden_target_name,
     tsi_ssl_session_cache* ssl_session_cache) {
   if (channel_creds == nullptr) {
-    gpr_log(GPR_ERROR,
-            "channel_creds is nullptr in "
-            "TlsChannelSecurityConnectorCreate()");
+    LOG(ERROR) << "channel_creds is nullptr in "
+                  "TlsChannelSecurityConnectorCreate()";
     return nullptr;
   }
   if (options == nullptr) {
-    gpr_log(GPR_ERROR,
-            "options is nullptr in "
-            "TlsChannelSecurityConnectorCreate()");
+    LOG(ERROR) << "options is nullptr in "
+                  "TlsChannelSecurityConnectorCreate()";
     return nullptr;
   }
   if (target_name == nullptr) {
-    gpr_log(GPR_ERROR,
-            "target_name is nullptr in "
-            "TlsChannelSecurityConnectorCreate()");
+    LOG(ERROR) << "target_name is nullptr in "
+                  "TlsChannelSecurityConnectorCreate()";
     return nullptr;
   }
   return MakeRefCounted<TlsChannelSecurityConnector>(
@@ -354,8 +351,8 @@ void TlsChannelSecurityConnector::add_handshakers(
         /*network_bio_buf_size=*/0,
         /*ssl_bio_buf_size=*/0, &tsi_hs);
     if (result != TSI_OK) {
-      gpr_log(GPR_ERROR, "Handshaker creation failed with error %s.",
-              tsi_result_to_string(result));
+      LOG(ERROR) << "Handshaker creation failed with error "
+                 << tsi_result_to_string(result);
     }
   }
   // If tsi_hs is null, this will add a failing handshaker.
@@ -400,9 +397,8 @@ void TlsChannelSecurityConnector::cancel_check_peer(
       if (it != pending_verifier_requests_.end()) {
         pending_verifier_request = it->second->request();
       } else {
-        gpr_log(GPR_INFO,
-                "TlsChannelSecurityConnector::cancel_check_peer: no "
-                "corresponding pending request found");
+        VLOG(2) << "TlsChannelSecurityConnector::cancel_check_peer: no "
+                   "corresponding pending request found";
       }
     }
     if (pending_verifier_request != nullptr) {
@@ -452,7 +448,7 @@ void TlsChannelSecurityConnector::TlsChannelCertificateWatcher::
   if (root_ready && identity_ready) {
     if (security_connector_->UpdateHandshakerFactoryLocked() !=
         GRPC_SECURITY_OK) {
-      gpr_log(GPR_ERROR, "Update handshaker factory failed.");
+      LOG(ERROR) << "Update handshaker factory failed.";
     }
   }
 }
@@ -462,14 +458,12 @@ void TlsChannelSecurityConnector::TlsChannelCertificateWatcher::
 void TlsChannelSecurityConnector::TlsChannelCertificateWatcher::OnError(
     grpc_error_handle root_cert_error, grpc_error_handle identity_cert_error) {
   if (!root_cert_error.ok()) {
-    gpr_log(GPR_ERROR,
-            "TlsChannelCertificateWatcher getting root_cert_error: %s",
-            StatusToString(root_cert_error).c_str());
+    LOG(ERROR) << "TlsChannelCertificateWatcher getting root_cert_error: "
+               << StatusToString(root_cert_error);
   }
   if (!identity_cert_error.ok()) {
-    gpr_log(GPR_ERROR,
-            "TlsChannelCertificateWatcher getting identity_cert_error: %s",
-            StatusToString(identity_cert_error).c_str());
+    LOG(ERROR) << "TlsChannelCertificateWatcher getting identity_cert_error: "
+               << StatusToString(identity_cert_error);
   }
 }
 
@@ -565,15 +559,13 @@ TlsServerSecurityConnector::CreateTlsServerSecurityConnector(
     RefCountedPtr<grpc_server_credentials> server_creds,
     RefCountedPtr<grpc_tls_credentials_options> options) {
   if (server_creds == nullptr) {
-    gpr_log(GPR_ERROR,
-            "server_creds is nullptr in "
-            "TlsServerSecurityConnectorCreate()");
+    LOG(ERROR) << "server_creds is nullptr in "
+                  "TlsServerSecurityConnectorCreate()";
     return nullptr;
   }
   if (options == nullptr) {
-    gpr_log(GPR_ERROR,
-            "options is nullptr in "
-            "TlsServerSecurityConnectorCreate()");
+    LOG(ERROR) << "options is nullptr in "
+                  "TlsServerSecurityConnectorCreate()";
     return nullptr;
   }
   return MakeRefCounted<TlsServerSecurityConnector>(std::move(server_creds),
@@ -633,8 +625,8 @@ void TlsServerSecurityConnector::add_handshakers(
         server_handshaker_factory_, /*network_bio_buf_size=*/0,
         /*ssl_bio_buf_size=*/0, &tsi_hs);
     if (result != TSI_OK) {
-      gpr_log(GPR_ERROR, "Handshaker creation failed with error %s.",
-              tsi_result_to_string(result));
+      LOG(ERROR) << "Handshaker creation failed with error "
+                 << tsi_result_to_string(result);
     }
   }
   // If tsi_hs is null, this will add a failing handshaker.
@@ -679,9 +671,8 @@ void TlsServerSecurityConnector::cancel_check_peer(
       if (it != pending_verifier_requests_.end()) {
         pending_verifier_request = it->second->request();
       } else {
-        gpr_log(GPR_INFO,
-                "TlsServerSecurityConnector::cancel_check_peer: no "
-                "corresponding pending request found");
+        LOG(INFO) << "TlsServerSecurityConnector::cancel_check_peer: no "
+                     "corresponding pending request found";
       }
     }
     if (pending_verifier_request != nullptr) {
@@ -721,7 +712,7 @@ void TlsServerSecurityConnector::TlsServerCertificateWatcher::
       (!root_being_watched && identity_being_watched && identity_has_value)) {
     if (security_connector_->UpdateHandshakerFactoryLocked() !=
         GRPC_SECURITY_OK) {
-      gpr_log(GPR_ERROR, "Update handshaker factory failed.");
+      LOG(ERROR) << "Update handshaker factory failed.";
     }
   }
 }
@@ -731,14 +722,12 @@ void TlsServerSecurityConnector::TlsServerCertificateWatcher::
 void TlsServerSecurityConnector::TlsServerCertificateWatcher::OnError(
     grpc_error_handle root_cert_error, grpc_error_handle identity_cert_error) {
   if (!root_cert_error.ok()) {
-    gpr_log(GPR_ERROR,
-            "TlsServerCertificateWatcher getting root_cert_error: %s",
-            StatusToString(root_cert_error).c_str());
+    LOG(ERROR) << "TlsServerCertificateWatcher getting root_cert_error: "
+               << StatusToString(root_cert_error);
   }
   if (!identity_cert_error.ok()) {
-    gpr_log(GPR_ERROR,
-            "TlsServerCertificateWatcher getting identity_cert_error: %s",
-            StatusToString(identity_cert_error).c_str());
+    LOG(ERROR) << "TlsServerCertificateWatcher getting identity_cert_error: "
+               << StatusToString(identity_cert_error);
   }
 }
 
