@@ -28,6 +28,7 @@
 #include <utility>
 
 #include "absl/functional/any_invocable.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
@@ -35,11 +36,9 @@
 #include <grpc/impl/connectivity_state.h>
 #include <grpc/slice.h>
 #include <grpc/status.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
-#include "src/core/lib/channel/context.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted.h"
@@ -55,6 +54,7 @@
 #include "src/core/lib/promise/pipe.h"
 #include "src/core/lib/resource_quota/arena.h"
 #include "src/core/lib/slice/slice_buffer.h"
+#include "src/core/lib/transport/call_destination.h"
 #include "src/core/lib/transport/call_final_info.h"
 #include "src/core/lib/transport/call_spine.h"
 #include "src/core/lib/transport/connectivity_state.h"
@@ -163,8 +163,6 @@ using NextPromiseFactory =
 // for a stream.
 typedef struct grpc_stream grpc_stream;
 
-extern grpc_core::DebugOnlyTraceFlag grpc_trace_stream_refcount;
-
 typedef struct grpc_stream_refcount {
   grpc_core::RefCount refs;
   grpc_closure destroy;
@@ -192,9 +190,9 @@ void grpc_stream_ref_init(grpc_stream_refcount* refcount, int initial_refs,
 #ifndef NDEBUG
 inline void grpc_stream_ref(grpc_stream_refcount* refcount,
                             const char* reason) {
-  if (grpc_trace_stream_refcount.enabled()) {
-    gpr_log(GPR_DEBUG, "%s %p:%p REF %s", refcount->object_type, refcount,
-            refcount->destroy.cb_arg, reason);
+  if (GRPC_TRACE_FLAG_ENABLED(stream_refcount)) {
+    VLOG(2) << refcount->object_type << " " << refcount << ":"
+            << refcount->destroy.cb_arg << " REF " << reason;
   }
   refcount->refs.RefNonZero(DEBUG_LOCATION, reason);
 }
@@ -209,9 +207,9 @@ void grpc_stream_destroy(grpc_stream_refcount* refcount);
 #ifndef NDEBUG
 inline void grpc_stream_unref(grpc_stream_refcount* refcount,
                               const char* reason) {
-  if (grpc_trace_stream_refcount.enabled()) {
-    gpr_log(GPR_DEBUG, "%s %p:%p UNREF %s", refcount->object_type, refcount,
-            refcount->destroy.cb_arg, reason);
+  if (GRPC_TRACE_FLAG_ENABLED(stream_refcount)) {
+    VLOG(2) << refcount->object_type << " " << refcount << ":"
+            << refcount->destroy.cb_arg << " UNREF " << reason;
   }
   if (GPR_UNLIKELY(refcount->refs.Unref(DEBUG_LOCATION, reason))) {
     grpc_stream_destroy(refcount);
@@ -318,9 +316,6 @@ struct grpc_transport_stream_op_batch {
 };
 
 struct grpc_transport_stream_op_batch_payload {
-  explicit grpc_transport_stream_op_batch_payload(
-      grpc_call_context_element* context)
-      : context(context) {}
   struct {
     grpc_metadata_batch* send_initial_metadata = nullptr;
   } send_initial_metadata;
@@ -410,9 +405,6 @@ struct grpc_transport_stream_op_batch_payload {
     // initial metadata.
     bool tarpit = false;
   } cancel_stream;
-
-  // Indexes correspond to grpc_context_index enum values
-  grpc_call_context_element* context;
 };
 
 /// Transport op: a set of operations to perform on a transport as a whole
@@ -481,6 +473,15 @@ typedef struct grpc_transport_op {
   grpc_handler_private_op_data handler_private;
 } grpc_transport_op;
 
+// Allocate a grpc_transport_op, and preconfigure the on_complete closure to
+// \a on_complete and then delete the returned transport op
+grpc_transport_op* grpc_make_transport_op(grpc_closure* on_complete);
+// Allocate a grpc_transport_stream_op_batch, and preconfigure the on_complete
+// closure
+// to \a on_complete and then delete the returned transport op
+grpc_transport_stream_op_batch* grpc_make_transport_stream_op(
+    grpc_closure* on_complete);
+
 void grpc_transport_stream_op_batch_finish_with_failure(
     grpc_transport_stream_op_batch* batch, grpc_error_handle error,
     grpc_core::CallCombiner* call_combiner);
@@ -502,11 +503,28 @@ class FilterStackTransport;
 class ClientTransport;
 class ServerTransport;
 
-class Transport : public Orphanable {
+class Transport : public InternallyRefCounted<Transport> {
  public:
   struct RawPointerChannelArgTag {};
   static absl::string_view ChannelArgName() { return GRPC_ARG_TRANSPORT; }
 
+  using InternallyRefCounted<Transport>::InternallyRefCounted;
+
+  // Though internally ref counted transports expose their "Ref" method to
+  // create a RefCountedPtr to themselves. The OrphanablePtr owner is the
+  // singleton decision maker on whether the transport should be destroyed or
+  // not.
+  // TODO(ctiller): consider moving to a DualRefCounted model (with the
+  // disadvantage that we would accidentally have many strong owners which is
+  // unnecessary for this type).
+  RefCountedPtr<Transport> Ref() {
+    return InternallyRefCounted<Transport>::Ref();
+  }
+  template <typename T>
+  RefCountedPtr<T> RefAsSubclass() {
+    return InternallyRefCounted<Transport>::RefAsSubclass<T>();
+  }
+
   virtual FilterStackTransport* filter_stack_transport() = 0;
   virtual ClientTransport* client_transport() = 0;
   virtual ServerTransport* server_transport() = 0;
@@ -527,8 +545,19 @@ class Transport : public Orphanable {
   // implementation of grpc_transport_perform_op
   virtual void PerformOp(grpc_transport_op* op) = 0;
 
-  // implementation of grpc_transport_get_endpoint
-  virtual grpc_endpoint* GetEndpoint() = 0;
+  void StartConnectivityWatch(
+      OrphanablePtr<ConnectivityStateWatcherInterface> watcher) {
+    grpc_transport_op* op = grpc_make_transport_op(nullptr);
+    op->start_connectivity_watch = std::move(watcher);
+    PerformOp(op);
+  }
+
+  void DisconnectWithError(grpc_error_handle error) {
+    CHECK(!error.ok()) << error;
+    grpc_transport_op* op = grpc_make_transport_op(nullptr);
+    op->disconnect_with_error = error;
+    PerformOp(op);
+  }
 };
 
 class FilterStackTransport : public Transport {
@@ -574,6 +603,7 @@ class FilterStackTransport : public Transport {
 
 class ClientTransport : public Transport {
  public:
+  using Transport::Transport;
   virtual void StartCall(CallHandler call_handler) = 0;
 
  protected:
@@ -582,24 +612,10 @@ class ClientTransport : public Transport {
 
 class ServerTransport : public Transport {
  public:
-  // Acceptor helps transports create calls.
-  class Acceptor {
-   public:
-    // Returns an arena that can be used to allocate memory for initial metadata
-    // parsing, and later passed to CreateCall() as the underlying arena for
-    // that call.
-    virtual Arena* CreateArena() = 0;
-    // Create a call at the server (or fail)
-    // arena must have been previously allocated by CreateArena()
-    virtual absl::StatusOr<CallInitiator> CreateCall(
-        ClientMetadataHandle client_initial_metadata, Arena* arena) = 0;
-
-   protected:
-    ~Acceptor() = default;
-  };
-
+  using Transport::Transport;
   // Called once slightly after transport setup to register the accept function.
-  virtual void SetAcceptor(Acceptor* acceptor) = 0;
+  virtual void SetCallDestination(
+      RefCountedPtr<UnstartedCallDestination> unstarted_call_handler) = 0;
 
  protected:
   ~ServerTransport() override = default;
@@ -607,15 +623,6 @@ class ServerTransport : public Transport {
 
 }  // namespace grpc_core
 
-// Allocate a grpc_transport_op, and preconfigure the on_complete closure to
-// \a on_complete and then delete the returned transport op
-grpc_transport_op* grpc_make_transport_op(grpc_closure* on_complete);
-// Allocate a grpc_transport_stream_op_batch, and preconfigure the on_complete
-// closure
-// to \a on_complete and then delete the returned transport op
-grpc_transport_stream_op_batch* grpc_make_transport_stream_op(
-    grpc_closure* on_complete);
-
 namespace grpc_core {
 // This is the key to be used for loading/storing keepalive_throttling in the
 // absl::Status object.
