@@ -26,6 +26,7 @@
 #include <vector>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
@@ -36,7 +37,6 @@
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/impl/connectivity_state.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/channel/channel_args.h"
@@ -51,9 +51,6 @@
 #include "src/core/lib/gprpp/work_serializer.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/iomgr/pollset_set.h"
-#include "src/core/lib/json/json.h"
-#include "src/core/lib/json/json_args.h"
-#include "src/core/lib/json/json_object_loader.h"
 #include "src/core/lib/transport/connectivity_state.h"
 #include "src/core/load_balancing/address_filtering.h"
 #include "src/core/load_balancing/child_policy_handler.h"
@@ -62,11 +59,12 @@
 #include "src/core/load_balancing/lb_policy_factory.h"
 #include "src/core/load_balancing/lb_policy_registry.h"
 #include "src/core/resolver/endpoint_addresses.h"
+#include "src/core/util/json/json.h"
+#include "src/core/util/json/json_args.h"
+#include "src/core/util/json/json_object_loader.h"
 
 namespace grpc_core {
 
-TraceFlag grpc_lb_priority_trace(false, "priority_lb");
-
 namespace {
 
 using ::grpc_event_engine::experimental::EventEngine;
@@ -296,20 +294,20 @@ PriorityLb::PriorityLb(Args args)
           channel_args()
               .GetDurationFromIntMillis(GRPC_ARG_PRIORITY_FAILOVER_TIMEOUT_MS)
               .value_or(kDefaultChildFailoverTimeout))) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] created", this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this << "] created";
   }
 }
 
 PriorityLb::~PriorityLb() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] destroying priority LB policy", this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this << "] destroying priority LB policy";
   }
 }
 
 void PriorityLb::ShutdownLocked() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] shutting down", this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this << "] shutting down";
   }
   shutting_down_ = true;
   children_.clear();
@@ -318,10 +316,10 @@ void PriorityLb::ShutdownLocked() {
 void PriorityLb::ExitIdleLocked() {
   if (current_priority_ != UINT32_MAX) {
     const std::string& child_name = config_->priorities()[current_priority_];
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO,
-              "[priority_lb %p] exiting IDLE for current priority %d child %s",
-              this, current_priority_, child_name.c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << this
+                << "] exiting IDLE for current priority " << current_priority_
+                << " child " << child_name;
     }
     children_[child_name]->ExitIdleLocked();
   }
@@ -332,8 +330,8 @@ void PriorityLb::ResetBackoffLocked() {
 }
 
 absl::Status PriorityLb::UpdateLocked(UpdateArgs args) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] received update", this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this << "] received update";
   }
   // Update config.
   config_ = args.config.TakeAsSubclass<PriorityLbConfig>();
@@ -404,9 +402,9 @@ void PriorityLb::ChoosePriorityLocked() {
        ++priority) {
     // If the child for the priority does not exist yet, create it.
     const std::string& child_name = config_->priorities()[priority];
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO, "[priority_lb %p] trying priority %u, child %s", this,
-              priority, child_name.c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << this << "] trying priority " << priority
+                << ", child " << child_name;
     }
     auto& child = children_[child_name];
     // Create child if needed.
@@ -445,29 +443,27 @@ void PriorityLb::ChoosePriorityLocked() {
       return;
     }
     // Child has been failing for a while.  Move on to the next priority.
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO,
-              "[priority_lb %p] skipping priority %u, child %s: state=%s, "
-              "failover timer not pending",
-              this, priority, child_name.c_str(),
-              ConnectivityStateName(child->connectivity_state()));
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << this << "] skipping priority " << priority
+                << ", child " << child_name << ": state="
+                << ConnectivityStateName(child->connectivity_state())
+                << ", failover timer not pending";
     }
   }
   // If we didn't find any priority to try, pick the first one in state
   // CONNECTING.
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] no priority reachable, checking for CONNECTING "
-            "priority to delegate to",
-            this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this
+              << "] no priority reachable, checking for CONNECTING priority to "
+                 "delegate to";
   }
   for (uint32_t priority = 0; priority < config_->priorities().size();
        ++priority) {
     // If the child for the priority does not exist yet, create it.
     const std::string& child_name = config_->priorities()[priority];
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO, "[priority_lb %p] trying priority %u, child %s", this,
-              priority, child_name.c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << this << "] trying priority " << priority
+                << ", child " << child_name;
     }
     auto& child = children_[child_name];
     CHECK(child != nullptr);
@@ -486,12 +482,11 @@ void PriorityLb::ChoosePriorityLocked() {
 void PriorityLb::SetCurrentPriorityLocked(int32_t priority,
                                           bool deactivate_lower_priorities,
                                           const char* reason) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] selecting priority %u, child %s (%s, "
-            "deactivate_lower_priorities=%d)",
-            this, priority, config_->priorities()[priority].c_str(), reason,
-            deactivate_lower_priorities);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << this << "] selecting priority " << priority
+              << ", child " << config_->priorities()[priority] << " (" << reason
+              << ", deactivate_lower_priorities=" << deactivate_lower_priorities
+              << ")";
   }
   current_priority_ = priority;
   if (deactivate_lower_priorities) {
@@ -515,13 +510,11 @@ void PriorityLb::SetCurrentPriorityLocked(int32_t priority,
 PriorityLb::ChildPriority::DeactivationTimer::DeactivationTimer(
     RefCountedPtr<PriorityLb::ChildPriority> child_priority)
     : child_priority_(std::move(child_priority)) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] child %s (%p): deactivating -- will remove in "
-            "%" PRId64 "ms",
-            child_priority_->priority_policy_.get(),
-            child_priority_->name_.c_str(), child_priority_.get(),
-            kChildRetentionInterval.millis());
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << child_priority_->priority_policy_.get()
+              << "] child " << child_priority_->name_ << " ("
+              << child_priority_.get() << "): deactivating -- will remove in "
+              << kChildRetentionInterval.millis() << "ms";
   }
   timer_handle_ =
       child_priority_->priority_policy_->channel_control_helper()
@@ -539,10 +532,10 @@ PriorityLb::ChildPriority::DeactivationTimer::DeactivationTimer(
 
 void PriorityLb::ChildPriority::DeactivationTimer::Orphan() {
   if (timer_handle_.has_value()) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO, "[priority_lb %p] child %s (%p): reactivating",
-              child_priority_->priority_policy_.get(),
-              child_priority_->name_.c_str(), child_priority_.get());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << child_priority_->priority_policy_.get()
+                << "] child " << child_priority_->name_ << " ("
+                << child_priority_.get() << "): reactivating";
     }
     child_priority_->priority_policy_->channel_control_helper()
         ->GetEventEngine()
@@ -555,12 +548,11 @@ void PriorityLb::ChildPriority::DeactivationTimer::Orphan() {
 void PriorityLb::ChildPriority::DeactivationTimer::OnTimerLocked() {
   if (timer_handle_.has_value()) {
     timer_handle_.reset();
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO,
-              "[priority_lb %p] child %s (%p): deactivation timer fired, "
-              "deleting child",
-              child_priority_->priority_policy_.get(),
-              child_priority_->name_.c_str(), child_priority_.get());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << child_priority_->priority_policy_.get()
+                << "] child " << child_priority_->name_ << " ("
+                << child_priority_.get()
+                << "): deactivation timer fired, deleting child";
     }
     child_priority_->priority_policy_->DeleteChild(child_priority_.get());
   }
@@ -573,14 +565,13 @@ void PriorityLb::ChildPriority::DeactivationTimer::OnTimerLocked() {
 PriorityLb::ChildPriority::FailoverTimer::FailoverTimer(
     RefCountedPtr<PriorityLb::ChildPriority> child_priority)
     : child_priority_(std::move(child_priority)) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(
-        GPR_INFO,
-        "[priority_lb %p] child %s (%p): starting failover timer for %" PRId64
-        "ms",
-        child_priority_->priority_policy_.get(), child_priority_->name_.c_str(),
-        child_priority_.get(),
-        child_priority_->priority_policy_->child_failover_timeout_.millis());
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO)
+        << "[priority_lb " << child_priority_->priority_policy_.get()
+        << "] child " << child_priority_->name_ << " (" << child_priority_.get()
+        << "): starting failover timer for "
+        << child_priority_->priority_policy_->child_failover_timeout_.millis()
+        << "ms";
   }
   timer_handle_ =
       child_priority_->priority_policy_->channel_control_helper()
@@ -599,11 +590,10 @@ PriorityLb::ChildPriority::FailoverTimer::FailoverTimer(
 
 void PriorityLb::ChildPriority::FailoverTimer::Orphan() {
   if (timer_handle_.has_value()) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO,
-              "[priority_lb %p] child %s (%p): cancelling failover timer",
-              child_priority_->priority_policy_.get(),
-              child_priority_->name_.c_str(), child_priority_.get());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << child_priority_->priority_policy_.get()
+                << "] child " << child_priority_->name_ << " ("
+                << child_priority_.get() << "): cancelling failover timer";
     }
     child_priority_->priority_policy_->channel_control_helper()
         ->GetEventEngine()
@@ -616,12 +606,11 @@ void PriorityLb::ChildPriority::FailoverTimer::Orphan() {
 void PriorityLb::ChildPriority::FailoverTimer::OnTimerLocked() {
   if (timer_handle_.has_value()) {
     timer_handle_.reset();
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-      gpr_log(GPR_INFO,
-              "[priority_lb %p] child %s (%p): failover timer fired, "
-              "reporting TRANSIENT_FAILURE",
-              child_priority_->priority_policy_.get(),
-              child_priority_->name_.c_str(), child_priority_.get());
+    if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+      LOG(INFO) << "[priority_lb " << child_priority_->priority_policy_.get()
+                << "] child " << child_priority_->name_ << " ("
+                << child_priority_.get()
+                << "): failover timer fired, reporting TRANSIENT_FAILURE";
     }
     child_priority_->OnConnectivityStateUpdateLocked(
         GRPC_CHANNEL_TRANSIENT_FAILURE,
@@ -637,18 +626,18 @@ void PriorityLb::ChildPriority::FailoverTimer::OnTimerLocked() {
 PriorityLb::ChildPriority::ChildPriority(
     RefCountedPtr<PriorityLb> priority_policy, std::string name)
     : priority_policy_(std::move(priority_policy)), name_(std::move(name)) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] creating child %s (%p)",
-            priority_policy_.get(), name_.c_str(), this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get()
+              << "] creating child " << name_ << " (" << this << ")";
   }
   // Start the failover timer.
   failover_timer_ = MakeOrphanable<FailoverTimer>(Ref());
 }
 
 void PriorityLb::ChildPriority::Orphan() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] child %s (%p): orphaned",
-            priority_policy_.get(), name_.c_str(), this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get() << "] child "
+              << name_ << " (" << this << "): orphaned";
   }
   failover_timer_.reset();
   deactivation_timer_.reset();
@@ -676,9 +665,9 @@ absl::Status PriorityLb::ChildPriority::UpdateLocked(
     RefCountedPtr<LoadBalancingPolicy::Config> config,
     bool ignore_reresolution_requests) {
   if (priority_policy_->shutting_down_) return absl::OkStatus();
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO, "[priority_lb %p] child %s (%p): start update",
-            priority_policy_.get(), name_.c_str(), this);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get() << "] child "
+              << name_ << " (" << this << "): start update";
   }
   ignore_reresolution_requests_ = ignore_reresolution_requests;
   // Create policy if needed.
@@ -702,10 +691,10 @@ absl::Status PriorityLb::ChildPriority::UpdateLocked(
   update_args.resolution_note = priority_policy_->resolution_note_;
   update_args.args = priority_policy_->args_;
   // Update the policy.
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] child %s (%p): updating child policy handler %p",
-            priority_policy_.get(), name_.c_str(), this, child_policy_.get());
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get() << "] child "
+              << name_ << " (" << this << "): updating child policy handler "
+              << child_policy_.get();
   }
   return child_policy_->UpdateLocked(std::move(update_args));
 }
@@ -719,12 +708,11 @@ PriorityLb::ChildPriority::CreateChildPolicyLocked(const ChannelArgs& args) {
       std::make_unique<Helper>(this->Ref(DEBUG_LOCATION, "Helper"));
   OrphanablePtr<LoadBalancingPolicy> lb_policy =
       MakeOrphanable<ChildPolicyHandler>(std::move(lb_policy_args),
-                                         &grpc_lb_priority_trace);
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] child %s (%p): created new child policy "
-            "handler %p",
-            priority_policy_.get(), name_.c_str(), this, lb_policy.get());
+                                         &priority_lb_trace);
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get() << "] child "
+              << name_ << " (" << this << "): created new child policy handler "
+              << lb_policy.get();
   }
   // Add the parent's interested_parties pollset_set to that of the newly
   // created child policy. This will make the child policy progress upon
@@ -745,12 +733,11 @@ void PriorityLb::ChildPriority::ResetBackoffLocked() {
 void PriorityLb::ChildPriority::OnConnectivityStateUpdateLocked(
     grpc_connectivity_state state, const absl::Status& status,
     RefCountedPtr<SubchannelPicker> picker) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_lb_priority_trace)) {
-    gpr_log(GPR_INFO,
-            "[priority_lb %p] child %s (%p): state update: %s (%s) picker %p",
-            priority_policy_.get(), name_.c_str(), this,
-            ConnectivityStateName(state), status.ToString().c_str(),
-            picker.get());
+  if (GRPC_TRACE_FLAG_ENABLED(priority_lb)) {
+    LOG(INFO) << "[priority_lb " << priority_policy_.get() << "] child "
+              << name_ << " (" << this
+              << "): state update: " << ConnectivityStateName(state) << " ("
+              << status << ") picker " << picker.get();
   }
   // Store the state and picker.
   connectivity_state_ = state;
