@@ -22,11 +22,13 @@
 #include "absl/strings/str_cat.h"
 #include "absl/time/clock.h"
 #include "constants.h"
+#include "metadata_exchange.h"
 #include "observability_util.h"
 #include "python_observability_context.h"
 
 #include <grpc/slice.h>
 
+#include "src/core/lib/experiments/experiments.h"
 #include "src/core/lib/slice/slice.h"
 
 namespace grpc_observability {
@@ -42,10 +44,15 @@ constexpr uint32_t
 
 PythonOpenCensusCallTracer::PythonOpenCensusCallTracer(
     const char* method, const char* target, const char* trace_id,
-    const char* parent_span_id, bool tracing_enabled, bool registered_method)
+    const char* parent_span_id, const char* identifier,
+    const std::vector<Label>& exchange_labels, bool tracing_enabled,
+    bool add_csm_optional_labels, bool registered_method)
     : method_(GetMethod(method)),
       target_(GetTarget(target)),
       tracing_enabled_(tracing_enabled),
+      add_csm_optional_labels_(add_csm_optional_labels),
+      labels_injector_(exchange_labels),
+      identifier_(identifier),
       registered_method_(registered_method) {
   GenerateClientContext(absl::StrCat("Sent.", method_),
                         absl::string_view(trace_id),
@@ -81,16 +88,17 @@ void PythonOpenCensusCallTracer::RecordAnnotation(
 
 PythonOpenCensusCallTracer::~PythonOpenCensusCallTracer() {
   if (PythonCensusStatsEnabled()) {
-    context_.Labels().emplace_back(kClientMethod, std::string(method_));
+    context_.Labels().emplace_back(kClientMethod, method_);
     RecordIntMetric(kRpcClientRetriesPerCallMeasureName, retries_ - 1,
-                    registered_method_,
-                    context_.Labels());  // exclude first attempt
+                    context_.Labels(), identifier_, registered_method_,
+                    /*include_exchange_labels=*/true);  // exclude first attempt
     RecordIntMetric(kRpcClientTransparentRetriesPerCallMeasureName,
-                    transparent_retries_, registered_method_,
-                    context_.Labels());
+                    transparent_retries_, context_.Labels(), identifier_,
+                    registered_method_, /*include_exchange_labels=*/true);
     RecordDoubleMetric(kRpcClientRetryDelayPerCallMeasureName,
-                       ToDoubleSeconds(retry_delay_), registered_method_,
-                       context_.Labels());
+                       ToDoubleSeconds(retry_delay_), context_.Labels(),
+                       identifier_, registered_method_,
+                       /*include_exchange_labels=*/true);
   }
 
   if (tracing_enabled_) {
@@ -151,10 +159,11 @@ PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
   if (!PythonCensusStatsEnabled()) {
     return;
   }
-  context_.Labels().emplace_back(kClientMethod, std::string(parent_->method_));
-  context_.Labels().emplace_back(kClientTarget, std::string(parent_->target_));
-  RecordIntMetric(kRpcClientStartedRpcsMeasureName, 1,
-                  parent_->registered_method_, context_.Labels());
+  context_.Labels().emplace_back(kClientMethod, parent_->method_);
+  context_.Labels().emplace_back(kClientTarget, parent_->target_);
+  RecordIntMetric(kRpcClientStartedRpcsMeasureName, 1, context_.Labels(),
+                  parent_->identifier_, parent_->registered_method_,
+                  /*include_exchange_labels=*/false);
 }
 
 void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
@@ -178,6 +187,19 @@ void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
     send_initial_metadata->Set(grpc_core::GrpcTagsBinMetadata(),
                                grpc_core::Slice(tags));
   }
+  parent_->labels_injector_.AddExchangeLabelsToMetadata(send_initial_metadata);
+}
+
+void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
+    RecordReceivedInitialMetadata(grpc_metadata_batch* recv_initial_metadata) {
+  if (recv_initial_metadata != nullptr &&
+      recv_initial_metadata->get(grpc_core::GrpcTrailersOnly())
+          .value_or(false)) {
+    is_trailers_only_ = true;
+    return;
+  }
+  labels_from_peer_ =
+      parent_->labels_injector_.GetExchangeLabels(recv_initial_metadata);
 }
 
 void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
@@ -195,6 +217,12 @@ std::shared_ptr<grpc_core::TcpTracerInterface> PythonOpenCensusCallTracer::
   return nullptr;
 }
 
+void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
+    SetOptionalLabel(OptionalLabelKey key,
+                     grpc_core::RefCountedStringValue value) {
+  optional_labels_array_[static_cast<size_t>(key)] = std::move(value);
+}
+
 namespace {
 
 // Returns 0 if no server stats are present in the metadata.
@@ -225,6 +253,10 @@ void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
   if (!PythonCensusStatsEnabled()) {
     return;
   }
+  if (is_trailers_only_) {
+    labels_from_peer_ =
+        parent_->labels_injector_.GetExchangeLabels(recv_trailing_metadata);
+  }
   auto status_code_ = status.code();
   uint64_t elapsed_time = 0;
   if (recv_trailing_metadata != nullptr) {
@@ -232,29 +264,56 @@ void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
   }
 
   std::string final_status = absl::StatusCodeToString(status_code_);
-  context_.Labels().emplace_back(kClientMethod, std::string(parent_->method_));
-  context_.Labels().emplace_back(kClientTarget, std::string(parent_->target_));
+  context_.Labels().emplace_back(kClientMethod, parent_->method_);
+  context_.Labels().emplace_back(kClientTarget, parent_->target_);
   context_.Labels().emplace_back(kClientStatus, final_status);
-  RecordDoubleMetric(
-      kRpcClientSentBytesPerRpcMeasureName,
-      static_cast<double>(transport_stream_stats != nullptr
-                              ? transport_stream_stats->outgoing.data_bytes
-                              : 0),
-      parent_->registered_method_, context_.Labels());
-  RecordDoubleMetric(
-      kRpcClientReceivedBytesPerRpcMeasureName,
-      static_cast<double>(transport_stream_stats != nullptr
-                              ? transport_stream_stats->incoming.data_bytes
-                              : 0),
-      parent_->registered_method_, context_.Labels());
+  if (parent_->add_csm_optional_labels_) {
+    parent_->labels_injector_.AddXdsOptionalLabels(
+        /*is_client=*/true, optional_labels_array_, context_.Labels());
+  }
+  for (const auto& label : labels_from_peer_) {
+    context_.Labels().emplace_back(label);
+  }
+  uint64_t incoming_bytes = 0;
+  uint64_t outgoing_bytes = 0;
+  if (grpc_core::IsCallTracerInTransportEnabled()) {
+    incoming_bytes = incoming_bytes_.load();
+    outgoing_bytes = outgoing_bytes_.load();
+  } else if (transport_stream_stats != nullptr) {
+    incoming_bytes = transport_stream_stats->incoming.data_bytes;
+    outgoing_bytes = transport_stream_stats->outgoing.data_bytes;
+  }
+  RecordDoubleMetric(kRpcClientSentBytesPerRpcMeasureName,
+                     static_cast<double>(outgoing_bytes), context_.Labels(),
+                     parent_->identifier_, parent_->registered_method_,
+                     /*include_exchange_labels=*/true);
+  RecordDoubleMetric(kRpcClientReceivedBytesPerRpcMeasureName,
+                     static_cast<double>(incoming_bytes), context_.Labels(),
+                     parent_->identifier_, parent_->registered_method_,
+                     /*include_exchange_labels=*/true);
   RecordDoubleMetric(kRpcClientServerLatencyMeasureName,
                      absl::ToDoubleSeconds(absl::Nanoseconds(elapsed_time)),
-                     parent_->registered_method_, context_.Labels());
+                     context_.Labels(), parent_->identifier_,
+                     parent_->registered_method_,
+                     /*include_exchange_labels=*/true);
   RecordDoubleMetric(kRpcClientRoundtripLatencyMeasureName,
                      absl::ToDoubleSeconds(absl::Now() - start_time_),
-                     parent_->registered_method_, context_.Labels());
-  RecordIntMetric(kRpcClientCompletedRpcMeasureName, 1,
-                  parent_->registered_method_, context_.Labels());
+                     context_.Labels(), parent_->identifier_,
+                     parent_->registered_method_,
+                     /*include_exchange_labels=*/true);
+  RecordIntMetric(kRpcClientCompletedRpcMeasureName, 1, context_.Labels(),
+                  parent_->identifier_, parent_->registered_method_,
+                  /*include_exchange_labels=*/true);
+}
+
+void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
+    RecordIncomingBytes(const TransportByteSize& transport_byte_size) {
+  incoming_bytes_.fetch_add(transport_byte_size.data_bytes);
+}
+
+void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
+    RecordOutgoingBytes(const TransportByteSize& transport_byte_size) {
+  outgoing_bytes_.fetch_add(transport_byte_size.data_bytes);
 }
 
 void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
@@ -263,16 +322,17 @@ void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::
 void PythonOpenCensusCallTracer::PythonOpenCensusCallAttemptTracer::RecordEnd(
     const gpr_timespec& /*latency*/) {
   if (PythonCensusStatsEnabled()) {
-    context_.Labels().emplace_back(kClientMethod,
-                                   std::string(parent_->method_));
+    context_.Labels().emplace_back(kClientMethod, parent_->method_);
     context_.Labels().emplace_back(kClientStatus,
                                    StatusCodeToString(status_code_));
     RecordIntMetric(kRpcClientSentMessagesPerRpcMeasureName,
-                    sent_message_count_, parent_->registered_method_,
-                    context_.Labels());
+                    sent_message_count_, context_.Labels(),
+                    parent_->identifier_, parent_->registered_method_,
+                    /*include_exchange_labels=*/true);
     RecordIntMetric(kRpcClientReceivedMessagesPerRpcMeasureName,
-                    recv_message_count_, parent_->registered_method_,
-                    context_.Labels());
+                    recv_message_count_, context_.Labels(),
+                    parent_->identifier_, parent_->registered_method_,
+                    /*include_exchange_labels=*/true);
 
     grpc_core::MutexLock lock(&parent_->mu_);
     if (--parent_->num_active_rpcs_ == 0) {
