@@ -21,6 +21,7 @@
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/time/time.h"
@@ -31,7 +32,6 @@
 #include <grpc/impl/propagation_bits.h>
 #include <grpc/slice.h>
 #include <grpc/status.h>
-#include <grpc/support/log.h>
 #include <grpc/support/time.h>
 
 #include "src/core/lib/channel/channel_args.h"
@@ -45,7 +45,7 @@
 #include "test/core/test_util/test_config.h"
 
 void run_test(bool wait_for_ready) {
-  gpr_log(GPR_INFO, "TEST: wait_for_ready=%d", wait_for_ready);
+  LOG(INFO) << "TEST: wait_for_ready=" << wait_for_ready;
 
   grpc_init();
 
@@ -104,7 +104,7 @@ void run_test(bool wait_for_ready) {
   cqv.Expect(grpc_core::CqVerifier::tag(1), true);
   cqv.Verify();
 
-  gpr_log(GPR_INFO, "call status: %d", status);
+  LOG(INFO) << "call status: " << status;
   if (wait_for_ready) {
     CHECK_EQ(status, GRPC_STATUS_DEADLINE_EXCEEDED);
   } else {
