@@ -35,9 +35,9 @@
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/gprpp/time.h"
-#include "src/core/lib/json/json.h"
 #include "src/core/load_balancing/lb_policy.h"
 #include "src/core/load_balancing/lb_policy_factory.h"
+#include "src/core/util/json/json.h"
 #include "test/core/end2end/end2end_tests.h"
 #include "test/core/test_util/test_lb_policies.h"
 
