@@ -36,16 +36,16 @@
 #include <sys/socket.h>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/time.h>
 
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/strerror.h"
 #include "src/core/lib/gprpp/thd.h"
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/socket_utils_posix.h"
+#include "src/core/util/useful.h"
 #include "test/core/test_util/cmdline.h"
 #include "test/core/test_util/histogram.h"
 
@@ -84,8 +84,7 @@ static int read_bytes(int fd, char* buf, size_t read_size, int spin) {
         if (errno == EAGAIN && spin == 1) {
           continue;
         }
-        gpr_log(GPR_ERROR, "Read failed: %s",
-                grpc_core::StrError(errno).c_str());
+        LOG(ERROR) << "Read failed: " << grpc_core::StrError(errno);
         return -1;
       }
     } else {
@@ -118,8 +117,7 @@ static int poll_read_bytes(int fd, char* buf, size_t read_size, int spin) {
       if (errno == EINTR) {
         continue;
       } else {
-        gpr_log(GPR_ERROR, "Poll failed: %s",
-                grpc_core::StrError(errno).c_str());
+        LOG(ERROR) << "Poll failed: " << grpc_core::StrError(errno);
         return -1;
       }
     }
@@ -130,7 +128,7 @@ static int poll_read_bytes(int fd, char* buf, size_t read_size, int spin) {
       err2 = read(fd, buf + bytes_read, read_size - bytes_read);
     } while (err2 < 0 && errno == EINTR);
     if (err2 < 0 && errno != EAGAIN) {
-      gpr_log(GPR_ERROR, "Read failed: %s", grpc_core::StrError(errno).c_str());
+      LOG(ERROR) << "Read failed: " << grpc_core::StrError(errno);
       return -1;
     }
     bytes_read += static_cast<size_t>(err2);
@@ -159,8 +157,7 @@ static int epoll_read_bytes(struct thread_args* args, char* buf, int spin) {
     err = epoll_wait(args->epoll_fd, &ev, 1, spin ? 0 : -1);
     if (err < 0) {
       if (errno == EINTR) continue;
-      gpr_log(GPR_ERROR, "epoll_wait failed: %s",
-              grpc_core::StrError(errno).c_str());
+      LOG(ERROR) << "epoll_wait failed: " << grpc_core::StrError(errno);
       return -1;
     }
     if (err == 0 && spin) continue;
@@ -206,8 +203,7 @@ static int blocking_write_bytes(struct thread_args* args, char* buf) {
       if (errno == EINTR) {
         continue;
       } else {
-        gpr_log(GPR_ERROR, "Read failed: %s",
-                grpc_core::StrError(errno).c_str());
+        LOG(ERROR) << "Read failed: " << grpc_core::StrError(errno);
         return -1;
       }
     } else {
@@ -245,7 +241,7 @@ static int epoll_setup(thread_args* args) {
   set_socket_nonblocking(args);
   epoll_fd = epoll_create(1);
   if (epoll_fd < 0) {
-    gpr_log(GPR_ERROR, "epoll_create: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "epoll_create: " << grpc_core::StrError(errno);
     return -1;
   }
 
@@ -254,7 +250,7 @@ static int epoll_setup(thread_args* args) {
   ev.events = EPOLLIN | EPOLLET;
   ev.data.fd = args->fds.read_fd;
   if (epoll_ctl(epoll_fd, EPOLL_CTL_ADD, args->fds.read_fd, &ev) < 0) {
-    gpr_log(GPR_ERROR, "epoll_ctl: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "epoll_ctl: " << grpc_core::StrError(errno);
   }
   return 0;
 }
@@ -263,16 +259,16 @@ static int epoll_setup(thread_args* args) {
 static void server_thread(thread_args* args) {
   char* buf = static_cast<char*>(gpr_malloc(args->msg_size));
   if (args->setup(args) < 0) {
-    gpr_log(GPR_ERROR, "Setup failed");
+    LOG(ERROR) << "Setup failed";
   }
   for (;;) {
     if (args->read_bytes(args, buf) < 0) {
-      gpr_log(GPR_ERROR, "Server read failed");
+      LOG(ERROR) << "Server read failed";
       gpr_free(buf);
       return;
     }
     if (args->write_bytes(args, buf) < 0) {
-      gpr_log(GPR_ERROR, "Server write failed");
+      LOG(ERROR) << "Server write failed";
       gpr_free(buf);
       return;
     }
@@ -287,11 +283,11 @@ static void server_thread_wrap(void* arg) {
 static void print_histogram(grpc_histogram* histogram) {
   // TODO(klempner): Print more detailed information, such as detailed histogram
   // buckets
-  gpr_log(GPR_INFO, "latency (50/95/99/99.9): %f/%f/%f/%f",
-          grpc_histogram_percentile(histogram, 50),
-          grpc_histogram_percentile(histogram, 95),
-          grpc_histogram_percentile(histogram, 99),
-          grpc_histogram_percentile(histogram, 99.9));
+  LOG(INFO) << "latency (50/95/99/99.9): "
+            << grpc_histogram_percentile(histogram, 50) << "/"
+            << grpc_histogram_percentile(histogram, 95) << "/"
+            << grpc_histogram_percentile(histogram, 99) << "/"
+            << grpc_histogram_percentile(histogram, 99.9);
 }
 
 static double now(void) {
@@ -310,16 +306,16 @@ static void client_thread(thread_args* args) {
   int i;
 
   if (args->setup(args) < 0) {
-    gpr_log(GPR_ERROR, "Setup failed");
+    LOG(ERROR) << "Setup failed";
   }
   for (i = 0; i < kNumIters; ++i) {
     start_time = now();
     if (args->write_bytes(args, buf) < 0) {
-      gpr_log(GPR_ERROR, "Client write failed");
+      LOG(ERROR) << "Client write failed";
       goto error;
     }
     if (args->read_bytes(args, buf) < 0) {
-      gpr_log(GPR_ERROR, "Client read failed");
+      LOG(ERROR) << "Client read failed";
       goto error;
     }
     end_time = now();
@@ -338,8 +334,7 @@ error:
 static int create_listening_socket(struct sockaddr* port, socklen_t len) {
   int fd = socket(port->sa_family, SOCK_STREAM, 0);
   if (fd < 0) {
-    gpr_log(GPR_ERROR, "Unable to create socket: %s",
-            grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "Unable to create socket: " << grpc_core::StrError(errno);
     goto error;
   }
 
@@ -357,17 +352,17 @@ static int create_listening_socket(struct sockaddr* port, socklen_t len) {
   }
 
   if (bind(fd, port, len) < 0) {
-    gpr_log(GPR_ERROR, "bind: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "bind: " << grpc_core::StrError(errno);
     goto error;
   }
 
   if (listen(fd, 1) < 0) {
-    gpr_log(GPR_ERROR, "listen: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "listen: " << grpc_core::StrError(errno);
     goto error;
   }
 
   if (getsockname(fd, port, &len) < 0) {
-    gpr_log(GPR_ERROR, "getsockname: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "getsockname: " << grpc_core::StrError(errno);
     goto error;
   }
 
@@ -384,8 +379,7 @@ static int connect_client(struct sockaddr* addr, socklen_t len) {
   int fd = socket(addr->sa_family, SOCK_STREAM, 0);
   int err;
   if (fd < 0) {
-    gpr_log(GPR_ERROR, "Unable to create socket: %s",
-            grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "Unable to create socket: " << grpc_core::StrError(errno);
     goto error;
   }
 
@@ -403,7 +397,7 @@ static int connect_client(struct sockaddr* addr, socklen_t len) {
   } while (err < 0 && errno == EINTR);
 
   if (err < 0) {
-    gpr_log(GPR_ERROR, "connect error: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "connect error: " << grpc_core::StrError(errno);
     goto error;
   }
   return fd;
@@ -418,7 +412,7 @@ error:
 static int accept_server(int listen_fd) {
   int fd = accept(listen_fd, nullptr, nullptr);
   if (fd < 0) {
-    gpr_log(GPR_ERROR, "Accept failed: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "Accept failed: " << grpc_core::StrError(errno);
     return -1;
   }
   return fd;
@@ -438,19 +432,19 @@ static int create_sockets_tcp(fd_pair* client_fds, fd_pair* server_fds) {
 
   listen_fd = create_listening_socket(sa_port, sizeof(port));
   if (listen_fd == -1) {
-    gpr_log(GPR_ERROR, "Listen failed");
+    LOG(ERROR) << "Listen failed";
     goto error;
   }
 
   client_fd = connect_client(sa_port, sizeof(port));
   if (client_fd == -1) {
-    gpr_log(GPR_ERROR, "Connect failed");
+    LOG(ERROR) << "Connect failed";
     goto error;
   }
 
   server_fd = accept_server(listen_fd);
   if (server_fd == -1) {
-    gpr_log(GPR_ERROR, "Accept failed");
+    LOG(ERROR) << "Accept failed";
     goto error;
   }
 
@@ -477,7 +471,7 @@ error:
 static int create_sockets_socketpair(fd_pair* client_fds, fd_pair* server_fds) {
   int fds[2];
   if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0) {
-    gpr_log(GPR_ERROR, "socketpair: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "socketpair: " << grpc_core::StrError(errno);
     return -1;
   }
 
@@ -492,12 +486,12 @@ static int create_sockets_pipe(fd_pair* client_fds, fd_pair* server_fds) {
   int cfds[2];
   int sfds[2];
   if (pipe(cfds) < 0) {
-    gpr_log(GPR_ERROR, "pipe: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "pipe: " << grpc_core::StrError(errno);
     return -1;
   }
 
   if (pipe(sfds) < 0) {
-    gpr_log(GPR_ERROR, "pipe: %s", grpc_core::StrError(errno).c_str());
+    LOG(ERROR) << "pipe: " << grpc_core::StrError(errno);
     return -1;
   }
 
@@ -592,8 +586,8 @@ static int run_benchmark(const char* socket_type, thread_args* client_args,
     return rv;
   }
 
-  gpr_log(GPR_INFO, "Starting test %s %s %zu", client_args->strategy_name,
-          socket_type, client_args->msg_size);
+  LOG(INFO) << "Starting test " << client_args->strategy_name << " "
+            << socket_type << " " << client_args->msg_size;
 
   grpc_core::Thread server("server_thread", server_thread_wrap, server_args);
   server.Start();
@@ -663,7 +657,7 @@ int main(int argc, char** argv) {
   }
 
   if (read_strategy == nullptr) {
-    gpr_log(GPR_INFO, "No strategy specified, running all benchmarks");
+    LOG(INFO) << "No strategy specified, running all benchmarks";
     return run_all_benchmarks(static_cast<size_t>(msg_size));
   }
 
