@@ -24,6 +24,9 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "absl/log/check.h"
+
+#include <grpc/credentials.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
 #include <grpc/support/string_util.h>
@@ -32,14 +35,13 @@
 #include "src/core/lib/config/config_vars.h"
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/gprpp/unique_type_name.h"
-#include "src/core/lib/iomgr/load_file.h"
 #include "src/core/lib/security/context/security_context.h"
 #include "src/core/lib/security/credentials/tls/grpc_tls_certificate_provider.h"
 #include "src/core/lib/security/credentials/tls/grpc_tls_credentials_options.h"
 #include "src/core/lib/security/credentials/tls/tls_credentials.h"
 #include "src/core/tsi/transport_security.h"
-#include "test/core/util/test_config.h"
-#include "test/core/util/tls_utils.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/tls_utils.h"
 
 #define CA_CERT_PATH "src/core/tsi/test_creds/ca.pem"
 #define CLIENT_CERT_PATH "src/core/tsi/test_creds/multi-domain.pem"
@@ -61,36 +63,14 @@ class TlsSecurityConnectorTest : public ::testing::Test {
   TlsSecurityConnectorTest() {}
 
   void SetUp() override {
-    grpc_slice ca_slice_1, ca_slice_0, cert_slice_1, key_slice_1, cert_slice_0,
-        key_slice_0;
-    GPR_ASSERT(GRPC_LOG_IF_ERROR("load_file",
-                                 grpc_load_file(CA_CERT_PATH, 1, &ca_slice_1)));
-    GPR_ASSERT(GRPC_LOG_IF_ERROR(
-        "load_file", grpc_load_file(CLIENT_CERT_PATH, 1, &ca_slice_0)));
-    GPR_ASSERT(GRPC_LOG_IF_ERROR(
-        "load_file", grpc_load_file(SERVER_CERT_PATH_1, 1, &cert_slice_1)));
-    GPR_ASSERT(GRPC_LOG_IF_ERROR(
-        "load_file", grpc_load_file(SERVER_KEY_PATH_1, 1, &key_slice_1)));
-    GPR_ASSERT(GRPC_LOG_IF_ERROR(
-        "load_file", grpc_load_file(SERVER_CERT_PATH_0, 1, &cert_slice_0)));
-    GPR_ASSERT(GRPC_LOG_IF_ERROR(
-        "load_file", grpc_load_file(SERVER_KEY_PATH_0, 1, &key_slice_0)));
-    root_cert_1_ = std::string(StringViewFromSlice(ca_slice_1));
-    root_cert_0_ = std::string(StringViewFromSlice(ca_slice_0));
-    std::string identity_key_1 = std::string(StringViewFromSlice(key_slice_1));
-    std::string identity_key_0 = std::string(StringViewFromSlice(key_slice_0));
-    std::string identity_cert_1 =
-        std::string(StringViewFromSlice(cert_slice_1));
-    std::string identity_cert_0 =
-        std::string(StringViewFromSlice(cert_slice_0));
-    identity_pairs_1_.emplace_back(identity_key_1, identity_cert_1);
-    identity_pairs_0_.emplace_back(identity_key_0, identity_cert_0);
-    grpc_slice_unref(ca_slice_1);
-    grpc_slice_unref(ca_slice_0);
-    grpc_slice_unref(cert_slice_1);
-    grpc_slice_unref(key_slice_1);
-    grpc_slice_unref(cert_slice_0);
-    grpc_slice_unref(key_slice_0);
+    root_cert_1_ = testing::GetFileContents(CA_CERT_PATH);
+    root_cert_0_ = testing::GetFileContents(CLIENT_CERT_PATH);
+    identity_pairs_1_.emplace_back(
+        testing::GetFileContents(SERVER_KEY_PATH_1),
+        testing::GetFileContents(SERVER_CERT_PATH_1));
+    identity_pairs_0_.emplace_back(
+        testing::GetFileContents(SERVER_KEY_PATH_0),
+        testing::GetFileContents(SERVER_CERT_PATH_0));
   }
 
   static void VerifyExpectedErrorCallback(void* arg, grpc_error_handle error) {
@@ -104,7 +84,7 @@ class TlsSecurityConnectorTest : public ::testing::Test {
 
   static std::string GetErrorMsg(grpc_error_handle error) {
     std::string error_str;
-    GPR_ASSERT(
+    CHECK(
         grpc_error_get_str(error, StatusStrProperty::kDescription, &error_str));
     return error_str;
   }
@@ -390,13 +370,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -426,13 +406,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -579,13 +559,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -615,13 +595,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -653,29 +633,29 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a full TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(7, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
-             TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_SECURITY_LEVEL_PEER_PROPERTY,
-                 tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
-                 &peer.properties[3]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
-                 &peer.properties[4]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[5]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
-                 &peer.properties[6]) == TSI_OK);
+  CHECK(tsi_construct_peer(7, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
+        TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_SECURITY_LEVEL_PEER_PROPERTY,
+            tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
+            &peer.properties[3]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
+            &peer.properties[4]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[5]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
+            &peer.properties[6]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -701,29 +681,29 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a full TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(7, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.com",
-                 &peer.properties[1]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
-             TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_SECURITY_LEVEL_PEER_PROPERTY,
-                 tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
-                 &peer.properties[3]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
-                 &peer.properties[4]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "*.com",
-                 &peer.properties[5]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
-                 &peer.properties[6]) == TSI_OK);
+  CHECK(tsi_construct_peer(7, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.com",
+            &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
+        TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_SECURITY_LEVEL_PEER_PROPERTY,
+            tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
+            &peer.properties[3]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
+            &peer.properties[4]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "*.com",
+            &peer.properties[5]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
+            &peer.properties[6]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: Hostname "
@@ -1010,13 +990,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -1041,13 +1021,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -1076,13 +1056,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -1109,13 +1089,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
