@@ -17,21 +17,23 @@
 
 #include <memory>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 #include "absl/container/flat_hash_map.h"
 #include "absl/functional/any_invocable.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
 #include "absl/types/span.h"
 #include "gmock/gmock.h"
 
-#include "src/core/lib/channel/call_tracer.h"
-#include "src/core/lib/channel/metrics.h"
 #include "src/core/lib/channel/promise_based_filter.h"
-#include "src/core/lib/channel/tcp_tracer.h"
 #include "src/core/lib/gprpp/ref_counted.h"
+#include "src/core/telemetry/call_tracer.h"
+#include "src/core/telemetry/metrics.h"
+#include "src/core/telemetry/tcp_tracer.h"
 
 namespace grpc_core {
 
@@ -85,6 +87,10 @@ class FakeClientCallTracer : public ClientCallTracer {
         const grpc_transport_stream_stats* /*transport_stream_stats*/)
         override {}
     void RecordEnd(const gpr_timespec& /*latency*/) override { Unref(); }
+    void RecordIncomingBytes(
+        const TransportByteSize& /*transport_byte_size*/) override {}
+    void RecordOutgoingBytes(
+        const TransportByteSize& /*transport_byte_size*/) override {}
     void RecordAnnotation(absl::string_view annotation) override {
       annotation_logger_->push_back(std::string(annotation));
     }
@@ -178,6 +184,10 @@ class FakeServerCallTracer : public ServerCallTracer {
   void RecordReceivedTrailingMetadata(
       grpc_metadata_batch* /*recv_trailing_metadata*/) override {}
   void RecordEnd(const grpc_call_final_info* /*final_info*/) override {}
+  void RecordIncomingBytes(
+      const TransportByteSize& /*transport_byte_size*/) override {}
+  void RecordOutgoingBytes(
+      const TransportByteSize& /*transport_byte_size*/) override {}
   void RecordAnnotation(absl::string_view annotation) override {
     annotation_logger_->push_back(std::string(annotation));
   }
@@ -208,15 +218,15 @@ class FakeStatsPlugin : public StatsPlugin {
           bool(const experimental::StatsPluginChannelScope& /*scope*/) const>
           channel_filter = nullptr,
       bool use_disabled_by_default_metrics = false)
-      : channel_filter_(std::move(channel_filter)) {
+      : channel_filter_(std::move(channel_filter)),
+        use_disabled_by_default_metrics_(use_disabled_by_default_metrics) {
     GlobalInstrumentsRegistry::ForEach(
         [&](const GlobalInstrumentsRegistry::GlobalInstrumentDescriptor&
                 descriptor) {
           if (!use_disabled_by_default_metrics &&
               !descriptor.enable_by_default) {
-            gpr_log(GPR_INFO,
-                    "FakeStatsPlugin[%p]: skipping disabled metric: %s", this,
-                    std::string(descriptor.name).c_str());
+            LOG(INFO) << "FakeStatsPlugin[" << this
+                      << "]: skipping disabled metric: " << descriptor.name;
             return;
           }
           switch (descriptor.instrument_type) {
@@ -268,10 +278,18 @@ class FakeStatsPlugin : public StatsPlugin {
       const ChannelArgs& /*args*/) const override {
     return {true, nullptr};
   }
+  std::shared_ptr<StatsPlugin::ScopeConfig> GetChannelScopeConfig(
+      const experimental::StatsPluginChannelScope& /*scope*/) const override {
+    return nullptr;
+  }
+  std::shared_ptr<StatsPlugin::ScopeConfig> GetServerScopeConfig(
+      const ChannelArgs& /*args*/) const override {
+    return nullptr;
+  }
 
   void AddCounter(
-      GlobalInstrumentsRegistry::GlobalUInt64CounterHandle handle,
-      uint64_t value, absl::Span<const absl::string_view> label_values,
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, uint64_t value,
+      absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) override {
     // The problem with this approach is that we initialize uint64_counters_ in
     // BuildAndRegister by querying the GlobalInstrumentsRegistry at the time.
@@ -282,69 +300,70 @@ class FakeStatsPlugin : public StatsPlugin {
     // just ignore it here. This would also prevent us from having to lock the
     // GlobalInstrumentsRegistry everytime a metric is recorded. But this is not
     // a concern for now.
-    gpr_log(GPR_INFO,
-            "FakeStatsPlugin[%p]::AddCounter(index=%u, value=(uint64)%lu, "
-            "label_values={%s}, optional_label_values={%s}",
-            this, handle.index, value,
-            absl::StrJoin(label_values, ", ").c_str(),
-            absl::StrJoin(optional_values, ", ").c_str());
+    LOG(INFO) << "FakeStatsPlugin[" << this
+              << "]::AddCounter(index=" << handle.index << ", value=(uint64)"
+              << value << ", label_values={"
+              << absl::StrJoin(label_values, ", ")
+              << "}, optional_label_values={"
+              << absl::StrJoin(optional_values, ", ") << "}";
     MutexLock lock(&mu_);
     auto iter = uint64_counters_.find(handle.index);
     if (iter == uint64_counters_.end()) return;
     iter->second.Add(value, label_values, optional_values);
   }
   void AddCounter(
-      GlobalInstrumentsRegistry::GlobalDoubleCounterHandle handle, double value,
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, double value,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) override {
-    gpr_log(GPR_INFO,
-            "FakeStatsPlugin[%p]::AddCounter(index=%u, value(double)=%f, "
-            "label_values={%s}, optional_label_values={%s}",
-            this, handle.index, value,
-            absl::StrJoin(label_values, ", ").c_str(),
-            absl::StrJoin(optional_values, ", ").c_str());
+    LOG(INFO) << "FakeStatsPlugin[" << this
+              << "]::AddCounter(index=" << handle.index
+              << ", value(double)=" << value << ", label_values={"
+              << absl::StrJoin(label_values, ", ")
+              << "}, optional_label_values={"
+              << absl::StrJoin(optional_values, ", ") << "}";
     MutexLock lock(&mu_);
     auto iter = double_counters_.find(handle.index);
     if (iter == double_counters_.end()) return;
     iter->second.Add(value, label_values, optional_values);
   }
   void RecordHistogram(
-      GlobalInstrumentsRegistry::GlobalUInt64HistogramHandle handle,
-      uint64_t value, absl::Span<const absl::string_view> label_values,
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, uint64_t value,
+      absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) override {
-    gpr_log(GPR_INFO,
-            "FakeStatsPlugin[%p]::RecordHistogram(index=%u, value=(uint64)%lu, "
-            "label_values={%s}, optional_label_values={%s}",
-            this, handle.index, value,
-            absl::StrJoin(label_values, ", ").c_str(),
-            absl::StrJoin(optional_values, ", ").c_str());
+    LOG(INFO) << "FakeStatsPlugin[" << this
+              << "]::RecordHistogram(index=" << handle.index
+              << ", value=(uint64)" << value << ", label_values={"
+              << absl::StrJoin(label_values, ", ")
+              << "}, optional_label_values={"
+              << absl::StrJoin(optional_values, ", ") << "}";
     MutexLock lock(&mu_);
     auto iter = uint64_histograms_.find(handle.index);
     if (iter == uint64_histograms_.end()) return;
     iter->second.Record(value, label_values, optional_values);
   }
   void RecordHistogram(
-      GlobalInstrumentsRegistry::GlobalDoubleHistogramHandle handle,
-      double value, absl::Span<const absl::string_view> label_values,
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, double value,
+      absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) override {
-    gpr_log(GPR_INFO,
-            "FakeStatsPlugin[%p]::RecordHistogram(index=%u, value=(double)%f, "
-            "label_values={%s}, optional_label_values={%s}",
-            this, handle.index, value,
-            absl::StrJoin(label_values, ", ").c_str(),
-            absl::StrJoin(optional_values, ", ").c_str());
+    LOG(INFO) << "FakeStatsPlugin[" << this
+              << "]::RecordHistogram(index=" << handle.index
+              << ", value=(double)" << value << ", label_values={"
+              << absl::StrJoin(label_values, ", ")
+              << "}, optional_label_values={"
+              << absl::StrJoin(optional_values, ", ") << "}";
     MutexLock lock(&mu_);
     auto iter = double_histograms_.find(handle.index);
     if (iter == double_histograms_.end()) return;
     iter->second.Record(value, label_values, optional_values);
   }
   void AddCallback(RegisteredMetricCallback* callback) override {
-    gpr_log(GPR_INFO, "FakeStatsPlugin[%p]::AddCallback(%p)", this, callback);
+    LOG(INFO) << "FakeStatsPlugin[" << this << "]::AddCallback(" << callback
+              << ")";
     callbacks_.insert(callback);
   }
   void RemoveCallback(RegisteredMetricCallback* callback) override {
-    gpr_log(GPR_INFO, "FakeStatsPlugin[%p]::RemoveCallback(%p)", this,
-            callback);
+    LOG(INFO) << "FakeStatsPlugin[" << this << "]::RemoveCallback(" << callback
+              << ")";
     callbacks_.erase(callback);
   }
 
@@ -357,9 +376,15 @@ class FakeStatsPlugin : public StatsPlugin {
       std::shared_ptr<StatsPlugin::ScopeConfig> /*scope_config*/) override {
     return nullptr;
   }
+  bool IsInstrumentEnabled(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle) const override {
+    const auto& descriptor =
+        GlobalInstrumentsRegistry::GetInstrumentDescriptor(handle);
+    return use_disabled_by_default_metrics_ || descriptor.enable_by_default;
+  }
 
-  absl::optional<uint64_t> GetCounterValue(
-      GlobalInstrumentsRegistry::GlobalUInt64CounterHandle handle,
+  absl::optional<uint64_t> GetUInt64CounterValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&mu_);
@@ -369,8 +394,8 @@ class FakeStatsPlugin : public StatsPlugin {
     }
     return iter->second.GetValue(label_values, optional_values);
   }
-  absl::optional<double> GetCounterValue(
-      GlobalInstrumentsRegistry::GlobalDoubleCounterHandle handle,
+  absl::optional<double> GetDoubleCounterValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&mu_);
@@ -380,8 +405,8 @@ class FakeStatsPlugin : public StatsPlugin {
     }
     return iter->second.GetValue(label_values, optional_values);
   }
-  absl::optional<std::vector<uint64_t>> GetHistogramValue(
-      GlobalInstrumentsRegistry::GlobalUInt64HistogramHandle handle,
+  absl::optional<std::vector<uint64_t>> GetUInt64HistogramValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&mu_);
@@ -391,8 +416,8 @@ class FakeStatsPlugin : public StatsPlugin {
     }
     return iter->second.GetValues(label_values, optional_values);
   }
-  absl::optional<std::vector<double>> GetHistogramValue(
-      GlobalInstrumentsRegistry::GlobalDoubleHistogramHandle handle,
+  absl::optional<std::vector<double>> GetDoubleHistogramValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&mu_);
@@ -403,15 +428,15 @@ class FakeStatsPlugin : public StatsPlugin {
     return iter->second.GetValues(label_values, optional_values);
   }
   void TriggerCallbacks() {
-    gpr_log(GPR_INFO, "FakeStatsPlugin[%p]::TriggerCallbacks(): START", this);
+    LOG(INFO) << "FakeStatsPlugin[" << this << "]::TriggerCallbacks(): START";
     Reporter reporter(*this);
     for (auto* callback : callbacks_) {
       callback->Run(reporter);
     }
-    gpr_log(GPR_INFO, "FakeStatsPlugin[%p]::TriggerCallbacks(): END", this);
+    LOG(INFO) << "FakeStatsPlugin[" << this << "]::TriggerCallbacks(): END";
   }
-  absl::optional<int64_t> GetCallbackGaugeValue(
-      GlobalInstrumentsRegistry::GlobalCallbackInt64GaugeHandle handle,
+  absl::optional<int64_t> GetInt64CallbackGaugeValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&callback_mu_);
@@ -421,8 +446,8 @@ class FakeStatsPlugin : public StatsPlugin {
     }
     return iter->second.GetValue(label_values, optional_values);
   }
-  absl::optional<double> GetCallbackGaugeValue(
-      GlobalInstrumentsRegistry::GlobalCallbackDoubleGaugeHandle handle,
+  absl::optional<double> GetDoubleCallbackGaugeValue(
+      GlobalInstrumentsRegistry::GlobalInstrumentHandle handle,
       absl::Span<const absl::string_view> label_values,
       absl::Span<const absl::string_view> optional_values) {
     MutexLock lock(&callback_mu_);
@@ -438,34 +463,32 @@ class FakeStatsPlugin : public StatsPlugin {
    public:
     explicit Reporter(FakeStatsPlugin& plugin) : plugin_(plugin) {}
 
-    void Report(
-        GlobalInstrumentsRegistry::GlobalCallbackInt64GaugeHandle handle,
-        int64_t value, absl::Span<const absl::string_view> label_values,
+    void ReportInt64(
+        GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, int64_t value,
+        absl::Span<const absl::string_view> label_values,
         absl::Span<const absl::string_view> optional_values) override {
-      gpr_log(GPR_INFO,
-              "FakeStatsPlugin[%p]::Reporter::Report(index=%u, "
-              "value=(uint64)%ld, label_values={%s}, "
-              "optional_label_values={%s}",
-              this, handle.index, value,
-              absl::StrJoin(label_values, ", ").c_str(),
-              absl::StrJoin(optional_values, ", ").c_str());
+      LOG(INFO) << "FakeStatsPlugin[" << this
+                << "]::Reporter::Report(index=" << handle.index
+                << ", value=(int64_t)" << value << ", label_values={"
+                << absl::StrJoin(label_values, ", ")
+                << "}, optional_label_values={"
+                << absl::StrJoin(optional_values, ", ") << "}";
       MutexLock lock(&plugin_.callback_mu_);
       auto iter = plugin_.int64_callback_gauges_.find(handle.index);
       if (iter == plugin_.int64_callback_gauges_.end()) return;
       iter->second.Set(value, label_values, optional_values);
     }
 
-    void Report(
-        GlobalInstrumentsRegistry::GlobalCallbackDoubleGaugeHandle handle,
-        double value, absl::Span<const absl::string_view> label_values,
+    void ReportDouble(
+        GlobalInstrumentsRegistry::GlobalInstrumentHandle handle, double value,
+        absl::Span<const absl::string_view> label_values,
         absl::Span<const absl::string_view> optional_values) override {
-      gpr_log(GPR_INFO,
-              "FakeStatsPlugin[%p]::Reporter::Report(index=%u, "
-              "value=(double)%f, label_values={%s}, "
-              "optional_label_values={%s}",
-              this, handle.index, value,
-              absl::StrJoin(label_values, ", ").c_str(),
-              absl::StrJoin(optional_values, ", ").c_str());
+      LOG(INFO) << "FakeStatsPlugin[" << this
+                << "]::Reporter::Report(index=" << handle.index
+                << ", value=(double)" << value << ", label_values={"
+                << absl::StrJoin(label_values, ", ")
+                << "}, optional_label_values={"
+                << absl::StrJoin(optional_values, ", ") << "}";
       MutexLock lock(&plugin_.callback_mu_);
       auto iter = plugin_.double_callback_gauges_.find(handle.index);
       if (iter == plugin_.double_callback_gauges_.end()) return;
@@ -600,6 +623,7 @@ class FakeStatsPlugin : public StatsPlugin {
   absl::AnyInvocable<bool(
       const experimental::StatsPluginChannelScope& /*scope*/) const>
       channel_filter_;
+  bool use_disabled_by_default_metrics_;
   // Instruments.
   Mutex mu_;
   absl::flat_hash_map<uint32_t, Counter<uint64_t>> uint64_counters_
@@ -654,19 +678,17 @@ class GlobalInstrumentsRegistryTestPeer {
  public:
   static void ResetGlobalInstrumentsRegistry();
 
-  static absl::optional<GlobalInstrumentsRegistry::GlobalUInt64CounterHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindUInt64CounterHandleByName(absl::string_view name);
-  static absl::optional<GlobalInstrumentsRegistry::GlobalDoubleCounterHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindDoubleCounterHandleByName(absl::string_view name);
-  static absl::optional<GlobalInstrumentsRegistry::GlobalUInt64HistogramHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindUInt64HistogramHandleByName(absl::string_view name);
-  static absl::optional<GlobalInstrumentsRegistry::GlobalDoubleHistogramHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindDoubleHistogramHandleByName(absl::string_view name);
-  static absl::optional<
-      GlobalInstrumentsRegistry::GlobalCallbackInt64GaugeHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindCallbackInt64GaugeHandleByName(absl::string_view name);
-  static absl::optional<
-      GlobalInstrumentsRegistry::GlobalCallbackDoubleGaugeHandle>
+  static absl::optional<GlobalInstrumentsRegistry::GlobalInstrumentHandle>
   FindCallbackDoubleGaugeHandleByName(absl::string_view name);
 
   static GlobalInstrumentsRegistry::GlobalInstrumentDescriptor*
