@@ -32,6 +32,7 @@
 #include <gmock/gmock.h>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/memory/memory.h"
 #include "absl/strings/str_cat.h"
 
@@ -40,14 +41,12 @@
 #include <grpc/grpc_security.h>
 #include <grpc/slice.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/string_util.h>
 #include <grpc/support/time.h>
 #include <grpcpp/impl/service_type.h>
 #include <grpcpp/server_builder.h>
 
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/gprpp/host_port.h"
 #include "src/core/lib/gprpp/thd.h"
@@ -56,6 +55,7 @@
 #include "src/core/lib/security/credentials/credentials.h"
 #include "src/core/lib/security/security_connector/alts/alts_security_connector.h"
 #include "src/core/lib/slice/slice_string_helpers.h"
+#include "src/core/util/useful.h"
 #include "test/core/end2end/cq_verifier.h"
 #include "test/core/test_util/fake_udp_and_tcp_server.h"
 #include "test/core/test_util/port.h"
@@ -113,8 +113,7 @@ class FakeHandshakeServer {
     // TODO(apolcyn): when removing the global concurrent handshake limiting
     // queue, set MAX_CONCURRENT_STREAMS on this server.
     server_ = builder.BuildAndStart();
-    gpr_log(GPR_INFO, "Fake handshaker server listening on %s",
-            address_.c_str());
+    LOG(INFO) << "Fake handshaker server listening on " << address_;
   }
 
   ~FakeHandshakeServer() {
@@ -148,13 +147,12 @@ class TestServer {
                                      server_creds));
     grpc_server_credentials_release(server_creds);
     grpc_server_start(server_);
-    gpr_log(GPR_DEBUG, "Start TestServer %p. listen on %s", this,
-            server_addr_.c_str());
+    VLOG(2) << "Start TestServer " << this << ". listen on " << server_addr_;
     server_thd_ = std::make_unique<std::thread>(PollUntilShutdown, this);
   }
 
   ~TestServer() {
-    gpr_log(GPR_DEBUG, "Begin dtor of TestServer %p", this);
+    VLOG(2) << "Begin dtor of TestServer " << this;
     grpc_server_shutdown_and_notify(server_, server_cq_, this);
     server_thd_->join();
     grpc_server_destroy(server_);
@@ -170,7 +168,7 @@ class TestServer {
         self->server_cq_, gpr_inf_future(GPR_CLOCK_REALTIME), nullptr);
     CHECK(ev.type == GRPC_OP_COMPLETE);
     CHECK(ev.tag == self);
-    gpr_log(GPR_DEBUG, "TestServer %p stop polling", self);
+    VLOG(2) << "TestServer " << self << " stop polling";
   }
 
  private:
@@ -210,7 +208,7 @@ class ConnectLoopRunner {
 
   static void ConnectLoop(const ConnectLoopRunner* self) {
     for (size_t i = 0; i < self->loops_; i++) {
-      gpr_log(GPR_DEBUG, "runner:%p connect_loop begin loop %ld", self, i);
+      VLOG(2) << "runner:" << self << " connect_loop begin loop " << i;
       grpc_completion_queue* cq =
           grpc_completion_queue_create_for_next(nullptr);
       grpc_channel* channel = create_secure_channel_for_test(
@@ -259,7 +257,7 @@ class ConnectLoopRunner {
       grpc_completion_queue_shutdown(cq);
       drain_cq(cq);
       grpc_completion_queue_destroy(cq);
-      gpr_log(GPR_DEBUG, "runner:%p connect_loop finished loop %ld", self, i);
+      VLOG(2) << "runner:" << self << " connect_loop finished loop " << i;
     }
   }
 
@@ -296,8 +294,7 @@ TEST(AltsConcurrentConnectivityTest, TestConcurrentClientServerHandshakes) {
     TestServer test_server;
     size_t num_concurrent_connects = 50;
     std::vector<std::unique_ptr<ConnectLoopRunner>> connect_loop_runners;
-    gpr_log(GPR_DEBUG,
-            "start performing concurrent expected-to-succeed connects");
+    VLOG(2) << "start performing concurrent expected-to-succeed connects";
     for (size_t i = 0; i < num_concurrent_connects; i++) {
       connect_loop_runners.push_back(std::make_unique<ConnectLoopRunner>(
           test_server.address(), fake_handshake_server.address(),
@@ -306,8 +303,7 @@ TEST(AltsConcurrentConnectivityTest, TestConcurrentClientServerHandshakes) {
           0 /* reconnect_backoff_ms unset */));
     }
     connect_loop_runners.clear();
-    gpr_log(GPR_DEBUG,
-            "done performing concurrent expected-to-succeed connects");
+    VLOG(2) << "done performing concurrent expected-to-succeed connects";
   }
 }
 
@@ -336,7 +332,7 @@ TEST(AltsConcurrentConnectivityTest,
   {
     std::vector<std::unique_ptr<ConnectLoopRunner>> connect_loop_runners;
     size_t num_concurrent_connects = 100;
-    gpr_log(GPR_DEBUG, "start performing concurrent expected-to-fail connects");
+    VLOG(2) << "start performing concurrent expected-to-fail connects";
     for (size_t i = 0; i < num_concurrent_connects; i++) {
       connect_loop_runners.push_back(std::make_unique<ConnectLoopRunner>(
           fake_backend_server.address(), fake_handshake_server.address(),
@@ -345,7 +341,7 @@ TEST(AltsConcurrentConnectivityTest,
           0 /* reconnect_backoff_ms unset */));
     }
     connect_loop_runners.clear();
-    gpr_log(GPR_DEBUG, "done performing concurrent expected-to-fail connects");
+    VLOG(2) << "done performing concurrent expected-to-fail connects";
   }
 }
 
@@ -368,7 +364,7 @@ TEST(AltsConcurrentConnectivityTest,
   {
     std::vector<std::unique_ptr<ConnectLoopRunner>> connect_loop_runners;
     size_t num_concurrent_connects = 100;
-    gpr_log(GPR_DEBUG, "start performing concurrent expected-to-fail connects");
+    VLOG(2) << "start performing concurrent expected-to-fail connects";
     for (size_t i = 0; i < num_concurrent_connects; i++) {
       connect_loop_runners.push_back(std::make_unique<ConnectLoopRunner>(
           fake_backend_server.address(), fake_handshake_server.address(),
@@ -377,7 +373,7 @@ TEST(AltsConcurrentConnectivityTest,
           0 /* reconnect_backoff_ms unset */));
     }
     connect_loop_runners.clear();
-    gpr_log(GPR_DEBUG, "done performing concurrent expected-to-fail connects");
+    VLOG(2) << "done performing concurrent expected-to-fail connects";
   }
 }
 
@@ -400,7 +396,7 @@ TEST(AltsConcurrentConnectivityTest,
   {
     std::vector<std::unique_ptr<ConnectLoopRunner>> connect_loop_runners;
     size_t num_concurrent_connects = 100;
-    gpr_log(GPR_DEBUG, "start performing concurrent expected-to-fail connects");
+    VLOG(2) << "start performing concurrent expected-to-fail connects";
     for (size_t i = 0; i < num_concurrent_connects; i++) {
       connect_loop_runners.push_back(std::make_unique<ConnectLoopRunner>(
           fake_backend_server.address(), fake_handshake_server.address(),
@@ -409,7 +405,7 @@ TEST(AltsConcurrentConnectivityTest,
           100 /* reconnect_backoff_ms */));
     }
     connect_loop_runners.clear();
-    gpr_log(GPR_DEBUG, "done performing concurrent expected-to-fail connects");
+    VLOG(2) << "done performing concurrent expected-to-fail connects";
   }
 }
 
