@@ -24,9 +24,9 @@
 #include <utility>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 
 #include <grpc/event_engine/event_engine.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/event_engine/default_event_engine.h"
@@ -50,9 +50,8 @@ FakeXdsTransportFactory::FakeStreamingCall::~FakeStreamingCall() {
     MutexLock lock(&mu_);
     if (transport_->abort_on_undrained_messages()) {
       for (const auto& message : from_client_messages_) {
-        gpr_log(GPR_ERROR, "[%s] %p From client message left in queue: %s",
-                transport_->server()->server_uri().c_str(), this,
-                message.c_str());
+        LOG(ERROR) << "[" << transport_->server()->server_uri() << "] " << this
+                   << " From client message left in queue: " << message;
       }
       CHECK(from_client_messages_.empty());
     }
