@@ -27,6 +27,7 @@
 
 #include "absl/cleanup/cleanup.h"
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/memory/memory.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
@@ -225,7 +226,7 @@ class BalancerServiceImpl : public BalancerService {
       shutdown_ = true;
     }
     ShutdownStream();
-    gpr_log(GPR_INFO, "LB[%p]: shut down", this);
+    LOG(INFO) << "LB[" << this << "]: shut down";
   }
 
   void set_client_load_reporting_interval_seconds(int seconds) {
@@ -285,11 +286,11 @@ class BalancerServiceImpl : public BalancerService {
  private:
   // Request handler.
   Status BalanceLoad(ServerContext* context, Stream* stream) override {
-    gpr_log(GPR_INFO, "LB[%p]: BalanceLoad", this);
+    LOG(INFO) << "LB[" << this << "]: BalanceLoad";
     {
       grpc_core::MutexLock lock(&mu_);
       if (shutdown_) {
-        gpr_log(GPR_INFO, "LB[%p]: shutdown at stream start", this);
+        LOG(INFO) << "LB[" << this << "]: shutdown at stream start";
         return Status::OK;
       }
     }
@@ -310,7 +311,7 @@ class BalancerServiceImpl : public BalancerService {
     // Read initial request.
     LoadBalanceRequest request;
     if (!stream->Read(&request)) {
-      gpr_log(GPR_INFO, "LB[%p]: stream read returned false", this);
+      LOG(INFO) << "LB[" << this << "]: stream read returned false";
       return Status::OK;
     }
     EXPECT_TRUE(request.has_initial_request());
@@ -319,8 +320,8 @@ class BalancerServiceImpl : public BalancerService {
       service_names_.push_back(request.initial_request().name());
     }
     IncreaseRequestCount();
-    gpr_log(GPR_INFO, "LB[%p]: received initial message '%s'", this,
-            request.DebugString().c_str());
+    LOG(INFO) << "LB[" << this << "]: received initial message '"
+              << request.DebugString() << "'";
     // Send initial response.
     LoadBalanceResponse response;
     auto* initial_response = response.mutable_initial_response();
@@ -334,11 +335,11 @@ class BalancerServiceImpl : public BalancerService {
     std::thread reader(std::bind(&BalancerServiceImpl::ReadThread, this, stream,
                                  &reader_shutdown));
     auto thread_cleanup = absl::MakeCleanup([&]() {
-      gpr_log(GPR_INFO, "shutting down reader thread");
+      LOG(INFO) << "shutting down reader thread";
       reader_shutdown.Notify();
-      gpr_log(GPR_INFO, "joining reader thread");
+      LOG(INFO) << "joining reader thread";
       reader.join();
-      gpr_log(GPR_INFO, "joining reader thread complete");
+      LOG(INFO) << "joining reader thread complete";
     });
     // Send responses as instructed by the test.
     while (true) {
@@ -347,12 +348,12 @@ class BalancerServiceImpl : public BalancerService {
         context->TryCancel();
         break;
       }
-      gpr_log(GPR_INFO, "LB[%p]: Sending response: %s", this,
-              response->DebugString().c_str());
+      LOG(INFO) << "LB[" << this
+                << "]: Sending response: " << response->DebugString();
       IncreaseResponseCount();
       stream->Write(*response);
     }
-    gpr_log(GPR_INFO, "LB[%p]: done", this);
+    LOG(INFO) << "LB[" << this << "]: done";
     return Status::OK;
   }
 
@@ -360,8 +361,8 @@ class BalancerServiceImpl : public BalancerService {
   void ReadThread(Stream* stream, absl::Notification* shutdown) {
     LoadBalanceRequest request;
     while (!shutdown->HasBeenNotified() && stream->Read(&request)) {
-      gpr_log(GPR_INFO, "LB[%p]: received client load report message '%s'",
-              this, request.DebugString().c_str());
+      LOG(INFO) << "LB[" << this << "]: received client load report message '"
+                << request.DebugString() << "'";
       EXPECT_GT(client_load_reporting_interval_seconds_, 0);
       EXPECT_TRUE(request.has_client_stats());
       ClientStats load_report;
@@ -451,7 +452,7 @@ class GrpclbEnd2endTest : public ::testing::Test {
     ~ServerThread() { Shutdown(); }
 
     void Start() {
-      gpr_log(GPR_INFO, "starting %s server on port %d", type_.c_str(), port_);
+      LOG(INFO) << "starting " << type_ << " server on port " << port_;
       CHECK(!running_);
       running_ = true;
       service_.Start();
@@ -463,7 +464,7 @@ class GrpclbEnd2endTest : public ::testing::Test {
       thread_ = std::make_unique<std::thread>(
           std::bind(&ServerThread::Serve, this, &mu, &cond));
       cond.Wait(&mu);
-      gpr_log(GPR_INFO, "%s server startup complete", type_.c_str());
+      LOG(INFO) << type_ << " server startup complete";
     }
 
     void Serve(grpc_core::Mutex* mu, grpc_core::CondVar* cond) {
@@ -481,11 +482,11 @@ class GrpclbEnd2endTest : public ::testing::Test {
 
     void Shutdown() {
       if (!running_) return;
-      gpr_log(GPR_INFO, "%s about to shutdown", type_.c_str());
+      LOG(INFO) << type_ << " about to shutdown";
       service_.Shutdown();
       server_->Shutdown(grpc_timeout_milliseconds_to_deadline(0));
       thread_->join();
-      gpr_log(GPR_INFO, "%s shutdown completed", type_.c_str());
+      LOG(INFO) << type_ << " shutdown completed";
       running_ = false;
     }
 
@@ -667,8 +668,8 @@ class GrpclbEnd2endTest : public ::testing::Test {
       size_t start_index = 0, size_t stop_index = 0,
       WaitForBackendOptions options = WaitForBackendOptions(),
       SourceLocation location = SourceLocation()) {
-    gpr_log(GPR_INFO, "Waiting for backends [%" PRIuPTR ", %" PRIuPTR ")",
-            start_index, stop_index);
+    LOG(INFO) << "Waiting for backends [" << start_index << ", " << stop_index
+              << ")";
     const absl::Time deadline =
         absl::Now() +
         absl::Seconds(options.timeout_seconds * grpc_test_slowdown_factor());
@@ -689,11 +690,11 @@ class GrpclbEnd2endTest : public ::testing::Test {
       SendRpcAndCount(&num_total, &num_ok, &num_failure, &num_drops);
     }
     ResetBackendCounters();
-    gpr_log(GPR_INFO,
-            "Performed %d warm up requests (a multiple of %d) against the "
-            "backends. %d succeeded, %d failed, %d dropped.",
-            num_total, options.num_requests_multiple_of, num_ok, num_failure,
-            num_drops);
+    LOG(INFO) << "Performed " << num_total
+              << " warm up requests (a multiple of "
+              << options.num_requests_multiple_of << ") against the backends. "
+              << num_ok << " succeeded, " << num_failure << " failed, "
+              << num_drops << " dropped.";
     return std::make_tuple(num_ok, num_failure, num_drops);
   }
 
@@ -1299,9 +1300,9 @@ TEST_F(GrpclbEnd2endTest,
   SetNextResolutionDefaultBalancer();
   WaitForBackend(0);
   // Send 10 requests.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // All 10 requests should have gone to the first backend.
   EXPECT_EQ(10U, backends_[0]->service().request_count());
   EXPECT_EQ(0U, backends_[1]->service().request_count());
@@ -1314,9 +1315,9 @@ TEST_F(GrpclbEnd2endTest,
   // Now tell the channel to use balancer 2.  However, the stream to the
   // default balancer is not terminated, so the client will continue to
   // use it.
-  gpr_log(GPR_INFO, "========= ABOUT TO UPDATE 1 ==========");
+  LOG(INFO) << "========= ABOUT TO UPDATE 1 ==========";
   SetNextResolution({balancer2->port()});
-  gpr_log(GPR_INFO, "========= UPDATE 1 DONE ==========");
+  LOG(INFO) << "========= UPDATE 1 DONE ==========";
   // Now the default balancer sends backend 2.
   SendBalancerResponse(BuildResponseForBackends({backends_[2]->port()}, {}));
   WaitForBackend(2);
@@ -1339,9 +1340,9 @@ TEST_F(GrpclbEnd2endTest,
   // Wait until the first backend is ready.
   WaitForBackend(0);
   // Send 10 requests.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // All 10 requests should have gone to the first backend.
   EXPECT_EQ(10U, backends_[0]->service().request_count());
   EXPECT_EQ(0U, backends_[1]->service().request_count());
@@ -1351,15 +1352,15 @@ TEST_F(GrpclbEnd2endTest,
   EXPECT_EQ(0U, balancer2->service().request_count());
   EXPECT_EQ(0U, balancer2->service().response_count());
   // Send another address list with the same list of balancers.
-  gpr_log(GPR_INFO, "========= ABOUT TO UPDATE 1 ==========");
+  LOG(INFO) << "========= ABOUT TO UPDATE 1 ==========";
   SetNextResolution({balancer_->port(), balancer2->port()});
-  gpr_log(GPR_INFO, "========= UPDATE 1 DONE ==========");
+  LOG(INFO) << "========= UPDATE 1 DONE ==========";
   // Shut down the balancer stream to force the client to create a new one.
   // The new stream should go to the default balancer, since the
   // underlying connection should not have been broken.
-  gpr_log(GPR_INFO, "========= SHUTTING DOWN BALANCER CALL ==========");
+  LOG(INFO) << "========= SHUTTING DOWN BALANCER CALL ==========";
   balancer_->service().ShutdownStream();
-  gpr_log(GPR_INFO, "========= DONE SHUTTING DOWN BALANCER CALL ==========");
+  LOG(INFO) << "========= DONE SHUTTING DOWN BALANCER CALL ==========";
   // Wait until client has created a new balancer stream.
   EXPECT_TRUE(balancer_->service().WaitForNewStream(1));
   // Make sure there was only one client connection seen by the balancer.
@@ -1383,34 +1384,34 @@ TEST_F(GrpclbEnd2endTest, BalancerDiesThenSwitchToNewBalancer) {
   EXPECT_EQ(0U, balancer2->service().request_count());
   EXPECT_EQ(0U, balancer2->service().response_count());
   // Send 10 RPCs.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // All 10 requests should have gone to the first backend.
   EXPECT_EQ(10U, backends_[0]->service().request_count());
   EXPECT_EQ(0U, backends_[1]->service().request_count());
   // Kill default balancer.
-  gpr_log(GPR_INFO, "********** ABOUT TO KILL BALANCER *************");
+  LOG(INFO) << "********** ABOUT TO KILL BALANCER *************";
   balancer_->Shutdown();
-  gpr_log(GPR_INFO, "********** KILLED BALANCER *************");
+  LOG(INFO) << "********** KILLED BALANCER *************";
   // Channel should continue using the last backend it saw from the
   // balancer before the balancer died.
-  gpr_log(GPR_INFO, "========= BEFORE SECOND BATCH ==========");
+  LOG(INFO) << "========= BEFORE SECOND BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH SECOND BATCH ==========");
+  LOG(INFO) << "========= DONE WITH SECOND BATCH ==========";
   // All 10 requests should again have gone to the first backend.
   EXPECT_EQ(20U, backends_[0]->service().request_count());
   EXPECT_EQ(0U, backends_[1]->service().request_count());
   // Tell channel to start using balancer 2.
-  gpr_log(GPR_INFO, "========= ABOUT TO UPDATE 1 ==========");
+  LOG(INFO) << "========= ABOUT TO UPDATE 1 ==========";
   SetNextResolution({balancer2->port()});
-  gpr_log(GPR_INFO, "========= UPDATE 1 DONE ==========");
+  LOG(INFO) << "========= UPDATE 1 DONE ==========";
   // Channel should start using backend 1.
   WaitForBackend(1);
   // This is serviced by the updated RR policy
-  gpr_log(GPR_INFO, "========= BEFORE THIRD BATCH ==========");
+  LOG(INFO) << "========= BEFORE THIRD BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH THIRD BATCH ==========");
+  LOG(INFO) << "========= DONE WITH THIRD BATCH ==========";
   // All 10 requests should have gone to the second backend.
   EXPECT_EQ(0U, backends_[0]->service().request_count());
   EXPECT_EQ(10U, backends_[1]->service().request_count());
@@ -1428,15 +1429,15 @@ TEST_F(GrpclbEnd2endTest, ReresolveDeadBackendWhileInFallback) {
   // responds, and a fallback backend.
   SetNextResolution({balancer_->port()}, {backends_[0]->port()});
   // Start servers and send 10 RPCs per server.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // All 10 requests should have gone to the fallback backend.
   EXPECT_EQ(10U, backends_[0]->service().request_count());
   // Kill backend 0.
-  gpr_log(GPR_INFO, "********** ABOUT TO KILL BACKEND 0 *************");
+  LOG(INFO) << "********** ABOUT TO KILL BACKEND 0 *************";
   backends_[0]->Shutdown();
-  gpr_log(GPR_INFO, "********** KILLED BACKEND 0 *************");
+  LOG(INFO) << "********** KILLED BACKEND 0 *************";
   // This should trigger re-resolution.
   EXPECT_TRUE(response_generator_->WaitForReresolutionRequest(
       absl::Seconds(5 * grpc_test_slowdown_factor())));
@@ -1446,9 +1447,9 @@ TEST_F(GrpclbEnd2endTest, ReresolveDeadBackendWhileInFallback) {
   // Wait until re-resolution has been seen, as signaled by the second backend
   // receiving a request.
   WaitForBackend(1);
-  gpr_log(GPR_INFO, "========= BEFORE SECOND BATCH ==========");
+  LOG(INFO) << "========= BEFORE SECOND BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH SECOND BATCH ==========");
+  LOG(INFO) << "========= DONE WITH SECOND BATCH ==========";
   // All 10 requests should have gone to the second backend.
   EXPECT_EQ(10U, backends_[1]->service().request_count());
   EXPECT_EQ(1U, balancer_->service().request_count());
@@ -1467,9 +1468,9 @@ TEST_F(GrpclbEnd2endTest, ReresolveWhenBalancerCallFails) {
   SetNextResolutionDefaultBalancer();
   WaitForBackend(0);
   // Send 10 RPCs.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   CheckRpcSendOk(10);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // All 10 requests should have gone to the first backend.
   EXPECT_EQ(10U, backends_[0]->service().request_count());
   // Balancer 0 got a single request and sent a single request.
@@ -1478,13 +1479,13 @@ TEST_F(GrpclbEnd2endTest, ReresolveWhenBalancerCallFails) {
   EXPECT_EQ(0U, balancer2->service().request_count());
   EXPECT_EQ(0U, balancer2->service().response_count());
   // Kill balancer 0.
-  gpr_log(GPR_INFO, "********** ABOUT TO KILL BALANCER 0 *************");
+  LOG(INFO) << "********** ABOUT TO KILL BALANCER 0 *************";
   balancer_->Shutdown();
-  gpr_log(GPR_INFO, "********** KILLED BALANCER 0 *************");
+  LOG(INFO) << "********** KILLED BALANCER 0 *************";
   // This should trigger a re-resolution.
   EXPECT_TRUE(response_generator_->WaitForReresolutionRequest(
       absl::Seconds(5 * grpc_test_slowdown_factor())));
-  gpr_log(GPR_INFO, "********** SAW RE-RESOLUTION REQUEST *************");
+  LOG(INFO) << "********** SAW RE-RESOLUTION REQUEST *************";
   // Re-resolution result switches to balancer 2.
   SetNextResolution({balancer2->port()});
   // Client should start using backend 1.
