@@ -28,9 +28,9 @@
 #include <gtest/gtest.h>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 
 #include <grpc/grpc.h>
-#include <grpc/support/log.h>
 #include <grpcpp/alarm.h>
 #include <grpcpp/security/credentials.h>
 #include <grpcpp/server_context.h>
@@ -98,35 +98,34 @@ class TestServiceSignaller {
   // is whatever the number of RPCs waiting for server notification is
   // at that time.
   int ClientWaitUntilNRpcsStarted(int desired_rpcs, absl::Duration timeout) {
-    gpr_log(GPR_DEBUG, "*** enter ClientWaitUntilNRpcsStarted ***");
+    VLOG(2) << "*** enter ClientWaitUntilNRpcsStarted ***";
     absl::Time deadline = absl::Now() + timeout;
     std::chrono::system_clock::time_point chrono_deadline =
         absl::ToChronoTime(deadline);
     std::unique_lock<std::mutex> lock(mu_);
     cv_rpc_started_.wait_until(lock, chrono_deadline, [this, desired_rpcs] {
-      gpr_log(
-          GPR_DEBUG,
-          "*** desired_rpcs: %d rpcs_waiting_for_server_to_continue_: %d ***",
-          desired_rpcs, rpcs_waiting_for_server_to_continue_);
+      VLOG(2) << "*** desired_rpcs: " << desired_rpcs
+              << " rpcs_waiting_for_server_to_continue_: "
+              << rpcs_waiting_for_server_to_continue_ << " ***";
       return rpcs_waiting_for_server_to_continue_ >= desired_rpcs;
     });
-    gpr_log(GPR_DEBUG, "*** leave ClientWaitUntilNRpcsStarted ***");
+    VLOG(2) << "*** leave ClientWaitUntilNRpcsStarted ***";
     return rpcs_waiting_for_server_to_continue_;
   }
   void ServerWaitToContinue() {
-    gpr_log(GPR_DEBUG, "*** enter ServerWaitToContinue ***");
+    VLOG(2) << "*** enter ServerWaitToContinue ***";
     std::unique_lock<std::mutex> lock(mu_);
     cv_server_continue_.wait(lock, [this] { return server_should_continue_; });
-    gpr_log(GPR_DEBUG, "*** leave ServerWaitToContinue ***");
+    VLOG(2) << "*** leave ServerWaitToContinue ***";
   }
   void SignalClientThatRpcStarted() {
-    gpr_log(GPR_DEBUG, "*** SignalClientThatRpcStarted ***");
+    VLOG(2) << "*** SignalClientThatRpcStarted ***";
     std::unique_lock<std::mutex> lock(mu_);
     ++rpcs_waiting_for_server_to_continue_;
     cv_rpc_started_.notify_all();
   }
   void SignalServerToContinue() {
-    gpr_log(GPR_DEBUG, "*** SignalServerToContinue ***");
+    VLOG(2) << "*** SignalServerToContinue ***";
     std::unique_lock<std::mutex> lock(mu_);
     server_should_continue_ = true;
     cv_server_continue_.notify_all();
@@ -170,7 +169,7 @@ class TestMultipleServiceImpl : public RpcService {
     }
 
     if (request->has_param() && request->param().server_die()) {
-      gpr_log(GPR_ERROR, "The request should not reach application handler.");
+      LOG(ERROR) << "The request should not reach application handler.";
       CHECK(0);
     }
     if (request->has_param() && request->param().has_expected_error()) {
@@ -257,7 +256,7 @@ class TestMultipleServiceImpl : public RpcService {
       }
     }
     if (request->has_param() &&
-        (request->param().expected_client_identity().length() > 0 ||
+        (!request->param().expected_client_identity().empty() ||
          request->param().check_auth_context())) {
       internal::CheckServerAuthContext(
           context, request->param().expected_transport_security_type(),
@@ -330,7 +329,7 @@ class TestMultipleServiceImpl : public RpcService {
     while (reader->Read(&request)) {
       response->mutable_message()->append(request.message());
     }
-    gpr_log(GPR_INFO, "Read: %d messages", num_msgs_read);
+    LOG(INFO) << "Read: " << num_msgs_read << " messages";
 
     if (server_try_cancel_thd != nullptr) {
       server_try_cancel_thd->join();
@@ -441,7 +440,7 @@ class TestMultipleServiceImpl : public RpcService {
     int read_counts = 0;
     while (stream->Read(&request)) {
       read_counts++;
-      gpr_log(GPR_INFO, "recv msg %s", request.message().c_str());
+      LOG(INFO) << "recv msg " << request.message();
       response.set_message(request.message());
       if (read_counts == server_write_last) {
         stream->WriteLast(response, WriteOptions());
