@@ -2,6 +2,7 @@ from bazel.cc_proto_descriptor_library.testdata import test1_pb2
 from bazel.cc_proto_descriptor_library.testdata import test_extension_pb2
 from bazel.cc_proto_descriptor_library.testdata import test_pb2
 from contrib.envoy.extensions.compression.qatzip.compressor.v3alpha import qatzip_pb2
+from contrib.envoy.extensions.compression.qatzstd.compressor.v3alpha import qatzstd_pb2
 from contrib.envoy.extensions.config.v3alpha import kv_store_xds_delegate_config_pb2
 from contrib.envoy.extensions.filters.http.checksum.v3alpha import checksum_pb2
 from contrib.envoy.extensions.filters.http.dynamo.v3 import dynamo_pb2
@@ -12,6 +13,7 @@ from contrib.envoy.extensions.filters.http.sxg.v3alpha import sxg_pb2
 from contrib.envoy.extensions.filters.network.client_ssl_auth.v3 import client_ssl_auth_pb2
 from contrib.envoy.extensions.filters.network.generic_proxy.action.v3 import action_pb2
 from contrib.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3 import dubbo_pb2
+from contrib.envoy.extensions.filters.network.generic_proxy.codecs.http1.v3 import http1_pb2
 from contrib.envoy.extensions.filters.network.generic_proxy.codecs.kafka.v3 import kafka_pb2
 from contrib.envoy.extensions.filters.network.generic_proxy.matcher.v3 import matcher_pb2
 from contrib.envoy.extensions.filters.network.generic_proxy.router.v3 import router_pb2
@@ -254,6 +256,7 @@ from envoy.data.cluster.v2alpha import outlier_detection_event_pb2
 from envoy.data.cluster.v3 import outlier_detection_event_pb2
 from envoy.data.core.v2alpha import health_check_event_pb2
 from envoy.data.core.v3 import health_check_event_pb2
+from envoy.data.core.v3 import tlv_metadata_pb2
 from envoy.data.dns.v2alpha import dns_table_pb2
 from envoy.data.dns.v3 import dns_table_pb2
 from envoy.data.tap.v2alpha import common_pb2
@@ -266,6 +269,7 @@ from envoy.data.tap.v3 import transport_pb2
 from envoy.data.tap.v3 import wrapper_pb2
 from envoy.extensions.access_loggers.file.v3 import file_pb2
 from envoy.extensions.access_loggers.filters.cel.v3 import cel_pb2
+from envoy.extensions.access_loggers.fluentd.v3 import fluentd_pb2
 from envoy.extensions.access_loggers.grpc.v3 import als_pb2
 from envoy.extensions.access_loggers.open_telemetry.v3 import logs_service_pb2
 from envoy.extensions.access_loggers.stream.v3 import stream_pb2
@@ -345,6 +349,7 @@ from envoy.extensions.filters.http.set_filter_state.v3 import set_filter_state_p
 from envoy.extensions.filters.http.set_metadata.v3 import set_metadata_pb2
 from envoy.extensions.filters.http.stateful_session.v3 import stateful_session_pb2
 from envoy.extensions.filters.http.tap.v3 import tap_pb2
+from envoy.extensions.filters.http.thrift_to_metadata.v3 import thrift_to_metadata_pb2
 from envoy.extensions.filters.http.upstream_codec.v3 import upstream_codec_pb2
 from envoy.extensions.filters.http.wasm.v3 import wasm_pb2
 from envoy.extensions.filters.listener.http_inspector.v3 import http_inspector_pb2
@@ -398,12 +403,12 @@ from envoy.extensions.http.custom_response.redirect_policy.v3 import redirect_po
 from envoy.extensions.http.early_header_mutation.header_mutation.v3 import header_mutation_pb2
 from envoy.extensions.http.header_formatters.preserve_case.v3 import preserve_case_pb2
 from envoy.extensions.http.header_validators.envoy_default.v3 import header_validator_pb2
+from envoy.extensions.http.injected_credentials.generic.v3 import generic_pb2
+from envoy.extensions.http.injected_credentials.oauth2.v3 import oauth2_pb2
 from envoy.extensions.http.original_ip_detection.custom_header.v3 import custom_header_pb2
 from envoy.extensions.http.original_ip_detection.xff.v3 import xff_pb2
 from envoy.extensions.http.stateful_session.cookie.v3 import cookie_pb2
 from envoy.extensions.http.stateful_session.header.v3 import header_pb2
-from envoy.extensions.injected_credentials.generic.v3 import generic_pb2
-from envoy.extensions.injected_credentials.oauth2.v3 import oauth2_pb2
 from envoy.extensions.internal_redirect.allow_listed_routes.v3 import allow_listed_routes_config_pb2
 from envoy.extensions.internal_redirect.previous_routes.v3 import previous_routes_config_pb2
 from envoy.extensions.internal_redirect.safe_cross_scheme.v3 import safe_cross_scheme_config_pb2
@@ -429,11 +434,15 @@ from envoy.extensions.network.dns_resolver.apple.v3 import apple_dns_resolver_pb
 from envoy.extensions.network.dns_resolver.cares.v3 import cares_dns_resolver_pb2
 from envoy.extensions.network.dns_resolver.getaddrinfo.v3 import getaddrinfo_dns_resolver_pb2
 from envoy.extensions.network.socket_interface.v3 import default_socket_interface_pb2
+from envoy.extensions.outlier_detection_monitors.common.v3 import error_types_pb2
+from envoy.extensions.outlier_detection_monitors.consecutive_errors.v3 import consecutive_errors_pb2
 from envoy.extensions.path.match.uri_template.v3 import uri_template_match_pb2
 from envoy.extensions.path.rewrite.uri_template.v3 import uri_template_rewrite_pb2
+from envoy.extensions.quic.connection_debug_visitor.v3 import connection_debug_visitor_basic_pb2
 from envoy.extensions.quic.connection_id_generator.v3 import envoy_deterministic_connection_id_generator_pb2
 from envoy.extensions.quic.crypto_stream.v3 import crypto_stream_pb2
 from envoy.extensions.quic.proof_source.v3 import proof_source_pb2
+from envoy.extensions.quic.server_preferred_address.v3 import datasource_pb2
 from envoy.extensions.quic.server_preferred_address.v3 import fixed_server_preferred_address_config_pb2
 from envoy.extensions.rate_limit_descriptors.expr.v3 import expr_pb2
 from envoy.extensions.rbac.audit_loggers.stream.v3 import stream_pb2
@@ -451,9 +460,12 @@ from envoy.extensions.router.cluster_specifiers.lua.v3 import lua_pb2
 from envoy.extensions.stat_sinks.graphite_statsd.v3 import graphite_statsd_pb2
 from envoy.extensions.stat_sinks.open_telemetry.v3 import open_telemetry_pb2
 from envoy.extensions.stat_sinks.wasm.v3 import wasm_pb2
+from envoy.extensions.string_matcher.lua.v3 import lua_pb2
 from envoy.extensions.tracers.opentelemetry.resource_detectors.v3 import dynatrace_resource_detector_pb2
 from envoy.extensions.tracers.opentelemetry.resource_detectors.v3 import environment_resource_detector_pb2
+from envoy.extensions.tracers.opentelemetry.resource_detectors.v3 import static_config_resource_detector_pb2
 from envoy.extensions.tracers.opentelemetry.samplers.v3 import always_on_sampler_pb2
+from envoy.extensions.tracers.opentelemetry.samplers.v3 import dynatrace_sampler_pb2
 from envoy.extensions.transport_sockets.alts.v3 import alts_pb2
 from envoy.extensions.transport_sockets.http_11_proxy.v3 import upstream_http_11_connect_pb2
 from envoy.extensions.transport_sockets.internal_upstream.v3 import internal_upstream_pb2
