@@ -53,7 +53,7 @@ def should_build_docker_interop_image_from_release_tag(lang):
 # Dictionary of default runtimes per language
 LANG_RUNTIME_MATRIX = {
     "cxx": ["cxx"],  # This is actually debian8.
-    "go": ["go1.8", "go1.11", "go1.16", "go1.19"],
+    "go": ["go1.x"],
     "java": ["java"],
     "python": ["python", "pythonasyncio"],
     "node": ["node"],
@@ -132,7 +132,8 @@ LANG_RELEASE_MATRIX = {
             ("v1.60.0", ReleaseInfo()),
             ("v1.61.0", ReleaseInfo()),
             ("v1.62.0", ReleaseInfo()),
-            ("v1.63.0", ReleaseInfo()),
+            ("v1.63.1", ReleaseInfo()),
+            ("v1.64.1", ReleaseInfo()),
         ]
     ),
     "go": OrderedDict(
@@ -299,7 +300,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.60.1", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.61.2", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.62.2", ReleaseInfo(runtimes=["go1.19"])),
-            ("v1.63.2", ReleaseInfo(runtimes=["go1.19"])),
+            ("v1.63.3", ReleaseInfo()),
+            ("v1.64.1", ReleaseInfo()),
+            ("v1.65.0", ReleaseInfo()),
         ]
     ),
     "java": OrderedDict(
@@ -441,7 +444,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.59.1", ReleaseInfo()),
             ("v1.60.1", ReleaseInfo()),
             ("v1.61.0", ReleaseInfo()),
-            ("v1.63.0", ReleaseInfo()),
+            ("v1.63.2", ReleaseInfo()),
+            ("v1.64.2", ReleaseInfo()),
+            ("v1.65.1", ReleaseInfo()),
         ]
     ),
     "python": OrderedDict(
@@ -788,6 +793,12 @@ LANG_RELEASE_MATRIX = {
                     runtimes=["python"], testcases_file="python__master"
                 ),
             ),
+            (
+                "v1.64.1",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
         ]
     ),
     "node": OrderedDict(
@@ -885,6 +896,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.61.0", ReleaseInfo()),
             ("v1.62.0", ReleaseInfo()),
             ("v1.63.0", ReleaseInfo()),
+            ("v1.64.0", ReleaseInfo()),
         ]
     ),
     "php": OrderedDict(
@@ -946,6 +958,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.61.0", ReleaseInfo()),
             ("v1.62.0", ReleaseInfo()),
             ("v1.63.0", ReleaseInfo()),
+            ("v1.64.0", ReleaseInfo()),
         ]
     ),
     "csharp": OrderedDict(
