@@ -136,7 +136,7 @@ pip_install() {
 
 pip_install --upgrade pip
 pip_install --upgrade wheel
-pip_install --upgrade setuptools==66.1.0
+pip_install --upgrade setuptools==70.1.1
 
 # pip-installs the directory specified. Used because on MSYS the vanilla Windows
 # Python gets confused when parsing paths.
@@ -158,8 +158,8 @@ pip_install_dir_and_deps() {
 
 pip_install -U gevent
 
-pip_install --upgrade 'cython<3.0.0rc1'
-pip_install --upgrade six 'protobuf>=4.21.3rc1,!=4.22.0.*'
+pip_install --upgrade 'cython>=3.0.0'
+pip_install --upgrade six 'protobuf>=5.26.1,<6.0dev'
 
 if [ "$("$VENV_PYTHON" -c "import sys; print(sys.version_info[0])")" == "2" ]
 then
@@ -178,6 +178,7 @@ if [ "$(is_mingw)" ] || [ "$(is_darwin)" ]; then
 else
   $VENV_PYTHON "$ROOT/src/python/grpcio_observability/make_grpcio_observability.py"
   pip_install_dir_and_deps "$ROOT/src/python/grpcio_observability"
+  pip_install_dir_and_deps "$ROOT/src/python/grpcio_csm_observability"
 fi
 
 # Build/install Channelz
@@ -215,9 +216,10 @@ pip_install_dir "$ROOT/src/python/grpcio_admin"
 pip_install_dir "$ROOT/src/python/grpcio_testing"
 
 # Build/install tests
+# shellcheck disable=SC2261
 pip_install coverage==7.2.0 oauth2client==4.1.0 \
             google-auth>=1.35.0 requests==2.31.0 \
-            googleapis-common-protos>=1.5.5 rsa==4.0 absl-py==1.4.0 \
+            rsa==4.0 absl-py==1.4.0 \
             opentelemetry-sdk==1.21.0
 $VENV_PYTHON "$ROOT/src/python/grpcio_tests/setup.py" preprocess
 $VENV_PYTHON "$ROOT/src/python/grpcio_tests/setup.py" build_package_protos
